#!/usr/bin/env python
"""
_ResourceControl_t_

Unit tests for ResourceControl.
"""

from future.utils import viewitems

import os
import subprocess
import sys
import threading
import unittest

from Utils.PythonVersion import PY3

from WMCore.Agent.Configuration import Configuration
from WMCore.BossAir.RunJob import RunJob
from WMCore.DAOFactory import DAOFactory
from WMCore.FwkJobReport.Report import Report
from WMCore.ResourceControl.ResourceControl import ResourceControl
from WMCore.Services.UUIDLib import makeUUID
from WMCore.WMBS.File import File
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.Job import Job
from WMCore.WMBS.JobGroup import JobGroup
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow import Workflow
from WMCore.WMBase import getTestBase
from WMQuality.Emulators.EmulatedUnitTestCase import EmulatedUnitTestCase
from WMQuality.TestInit import TestInit


class ResourceControlTest(EmulatedUnitTestCase):
    def setUp(self):
        """
        _setUp_

        Install schema and create a DAO factory for WMBS.
        """
        super(ResourceControlTest, self).setUp()
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules=["WMCore.WMBS",
                                               "WMCore.ResourceControl",
                                               "WMCore.BossAir"],
                                useDefault=False)

        myThread = threading.currentThread()
        self.daoFactory = DAOFactory(package="WMCore.WMBS",
                                     logger=myThread.logger,
                                     dbinterface=myThread.dbi)

        self.baDaoFactory = DAOFactory(package="WMCore.BossAir",
                                       logger=myThread.logger,
                                       dbinterface=myThread.dbi)

        self.insertRunJob = self.baDaoFactory(classname="NewJobs")
        self.insertState = self.baDaoFactory(classname="NewState")
        states = ['PEND', 'RUN', 'Idle', 'Running']
        self.insertState.execute(states)

        self.tempDir = self.testInit.generateWorkDir()
        return

    def tearDown(self):
        """
        _tearDown_

        Clear the schema.
        """
        super(ResourceControlTest, self).tearDown()
        self.testInit.clearDatabase()
        return

    def createJobs(self):
        """
        _createJobs_

        Create test jobs in WMBS and BossAir
        """
        testWorkflow = Workflow(spec=makeUUID(), owner="tapas",
                                name=makeUUID(), task="Test")
        testWorkflow.create()

        testFilesetA = Fileset(name="TestFilesetA")
        testFilesetA.create()
        testFilesetB = Fileset(name="TestFilesetB")
        testFilesetB.create()
        testFilesetC = Fileset(name="TestFilesetC")
        testFilesetC.create()

        testFileA = File(lfn="testFileA", locations=set(["testSE1", "testSE2"]))
        testFileA.create()
        testFilesetA.addFile(testFileA)
        testFilesetA.commit()
        testFilesetB.addFile(testFileA)
        testFilesetB.commit()
        testFilesetC.addFile(testFileA)
        testFilesetC.commit()

        testSubscriptionA = Subscription(fileset=testFilesetA,
                                         workflow=testWorkflow,
                                         type="Processing")
        testSubscriptionA.create()
        testSubscriptionA.addWhiteBlackList([{"site_name": "testSite1", "valid": True}])
        testSubscriptionB = Subscription(fileset=testFilesetB,
                                         workflow=testWorkflow,
                                         type="Processing")
        testSubscriptionB.create()
        testSubscriptionB.addWhiteBlackList([{"site_name": "testSite1", "valid": False}])
        testSubscriptionC = Subscription(fileset=testFilesetC,
                                         workflow=testWorkflow,
                                         type="Merge")
        testSubscriptionC.create()

        testJobGroupA = JobGroup(subscription=testSubscriptionA)
        testJobGroupA.create()
        testJobGroupB = JobGroup(subscription=testSubscriptionB)
        testJobGroupB.create()
        testJobGroupC = JobGroup(subscription=testSubscriptionC)
        testJobGroupC.create()

        # Site1, Has been assigned a location and is complete.
        testJobA = Job(name="testJobA", files=[testFileA])
        testJobA["couch_record"] = makeUUID()
        testJobA.create(group=testJobGroupA)
        testJobA["state"] = "success"

        # Site 1, Has been assigned a location and is incomplete.
        testJobB = Job(name="testJobB", files=[testFileA])
        testJobB["couch_record"] = makeUUID()
        testJobB["cache_dir"] = self.tempDir
        testJobB.create(group=testJobGroupA)
        testJobB["state"] = "executing"
        runJobB = RunJob()
        runJobB.buildFromJob(testJobB)
        runJobB["status"] = "PEND"

        # Does not have a location, white listed to site 1
        testJobC = Job(name="testJobC", files=[testFileA])
        testJobC["couch_record"] = makeUUID()
        testJobC.create(group=testJobGroupA)
        testJobC["state"] = "new"

        # Site 2, Has been assigned a location and is complete.
        testJobD = Job(name="testJobD", files=[testFileA])
        testJobD["couch_record"] = makeUUID()
        testJobD.create(group=testJobGroupB)
        testJobD["state"] = "success"

        # Site 2, Has been assigned a location and is incomplete.
        testJobE = Job(name="testJobE", files=[testFileA])
        testJobE["couch_record"] = makeUUID()
        testJobE.create(group=testJobGroupB)
        testJobE["state"] = "executing"
        runJobE = RunJob()
        runJobE.buildFromJob(testJobE)
        runJobE["status"] = "RUN"

        # Does not have a location, site 1 is blacklisted.
        testJobF = Job(name="testJobF", files=[testFileA])
        testJobF["couch_record"] = makeUUID()
        testJobF.create(group=testJobGroupB)
        testJobF["state"] = "new"

        # Site 3, Has been assigned a location and is complete.
        testJobG = Job(name="testJobG", files=[testFileA])
        testJobG["couch_record"] = makeUUID()
        testJobG.create(group=testJobGroupC)
        testJobG["state"] = "cleanout"

        # Site 3, Has been assigned a location and is incomplete.
        testJobH = Job(name="testJobH", files=[testFileA])
        testJobH["couch_record"] = makeUUID()
        testJobH.create(group=testJobGroupC)
        testJobH["state"] = "new"

        # Site 3, Does not have a location.
        testJobI = Job(name="testJobI", files=[testFileA])
        testJobI["couch_record"] = makeUUID()
        testJobI.create(group=testJobGroupC)
        testJobI["state"] = "new"

        # Site 3, Does not have a location and is in cleanout.
        testJobJ = Job(name="testJobJ", files=[testFileA])
        testJobJ["couch_record"] = makeUUID()
        testJobJ.create(group=testJobGroupC)
        testJobJ["state"] = "cleanout"

        changeStateAction = self.daoFactory(classname="Jobs.ChangeState")
        changeStateAction.execute(jobs=[testJobA, testJobB, testJobC, testJobD,
                                        testJobE, testJobF, testJobG, testJobH,
                                        testJobI, testJobJ])

        self.insertRunJob.execute([runJobB, runJobE])

        setLocationAction = self.daoFactory(classname="Jobs.SetLocation")
        setLocationAction.execute(testJobA["id"], "testSite1")
        setLocationAction.execute(testJobB["id"], "testSite1")
        setLocationAction.execute(testJobD["id"], "testSite1")
        setLocationAction.execute(testJobE["id"], "testSite2")
        setLocationAction.execute(testJobG["id"], "testSite1")
        setLocationAction.execute(testJobH["id"], "testSite1")

        return

    def testInsert(self):
        """
        _testInsert_

        Verify that inserting sites and thresholds works correctly, even if the
        site or threshold already exists.
        """
        myResourceControl = ResourceControl()
        myResourceControl.insertSite("testSite1", 10, 20, "testSE1", "testCE1")
        myResourceControl.insertSite("testSite1", 10, 20, "testSE1", "testCE1")
        myResourceControl.insertSite("testSite2", 100, 200, "testSE2", "testCE2")

        myResourceControl.insertThreshold("testSite1", "Processing", 20, 10)
        myResourceControl.insertThreshold("testSite1", "Merge", 200, 100)
        myResourceControl.insertThreshold("testSite1", "Merge", 250, 150)
        myResourceControl.insertThreshold("testSite2", "Processing", 50, 30)
        myResourceControl.insertThreshold("testSite2", "Merge", 135, 100)

        createThresholds = myResourceControl.listThresholdsForCreate()

        self.assertEqual(len(createThresholds), 2,
                         "Error: Wrong number of site in Resource Control DB")

        self.assertTrue("testSite1" in createThresholds,
                        "Error: Test Site 1 missing from thresholds.")

        self.assertTrue("testSite2" in createThresholds,
                        "Error: Test Site 2 missing from thresholds.")

        self.assertEqual(createThresholds["testSite1"]["total_slots"], 10,
                         "Error: Wrong number of total slots.")

        self.assertEqual(createThresholds["testSite1"]["pending_jobs"], {0: 0},
                         "Error: Wrong number of running jobs: %s" %
                         createThresholds["testSite1"]["pending_jobs"])

        self.assertEqual(createThresholds["testSite2"]["total_slots"], 100,
                         "Error: Wrong number of total slots.")

        self.assertEqual(createThresholds["testSite2"]["pending_jobs"], {0: 0},
                         "Error: Wrong number of running jobs.")

        thresholds = myResourceControl.listThresholdsForSubmit()

        self.assertEqual(len(thresholds), 2,
                         "Error: Wrong number of sites in Resource Control DB")

        self.assertTrue("testSite1" in thresholds,
                        "Error: testSite1 missing from thresholds.")

        self.assertTrue("testSite2" in thresholds,
                        "Error: testSite2 missing from thresholds.")

        site1Info = thresholds["testSite1"]
        site2Info = thresholds["testSite2"]
        site1Thresholds = site1Info["thresholds"]
        site2Thresholds = site2Info["thresholds"]

        procThreshold1 = None
        procThreshold2 = None
        mergeThreshold1 = None
        mergeThreshold2 = None
        for taskType, threshold in viewitems(site1Thresholds):
            if taskType == "Merge":
                mergeThreshold1 = threshold
            elif taskType == "Processing":
                procThreshold1 = threshold
        for taskType, threshold in viewitems(site2Thresholds):
            if taskType == "Merge":
                mergeThreshold2 = threshold
            elif taskType == "Processing":
                procThreshold2 = threshold

        self.assertEqual(len(site1Thresholds), 2,
                         "Error: Wrong number of task types.")

        self.assertEqual(len(site2Thresholds), 2,
                         "Error: Wrong number of task types.")

        self.assertNotEqual(procThreshold1, None)
        self.assertNotEqual(procThreshold2, None)
        self.assertNotEqual(mergeThreshold1, None)
        self.assertNotEqual(mergeThreshold2, None)

        self.assertEqual(site1Info["total_pending_slots"], 10,
                         "Error: Site thresholds wrong")

        self.assertEqual(site1Info["total_running_slots"], 20,
                         "Error: Site thresholds wrong")

        self.assertEqual(site1Info["total_running_jobs"], 0,
                         "Error: Site thresholds wrong")

        self.assertEqual(site1Info["total_pending_jobs"], 0,
                         "Error: Site thresholds wrong")

        self.assertEqual(procThreshold1["task_running_jobs"], 0,
                         "Error: Site thresholds wrong")

        self.assertEqual(procThreshold1["task_pending_jobs"], 0,
                         "Error: Site thresholds wrong")

        self.assertEqual(procThreshold1["max_slots"], 20,
                         "Error: Site thresholds wrong")

        self.assertEqual(procThreshold1["pending_slots"], 10,
                         "Error: Site thresholds wrong")

        self.assertEqual(mergeThreshold1["task_running_jobs"], 0,
                         "Error: Site thresholds wrong")

        self.assertEqual(mergeThreshold1["task_pending_jobs"], 0,
                         "Error: Site thresholds wrong")

        self.assertEqual(mergeThreshold1["max_slots"], 250,
                         "Error: Site thresholds wrong")

        self.assertEqual(mergeThreshold1["pending_slots"], 150,
                         "Error: Site thresholds wrong")

        self.assertEqual(site2Info["total_pending_slots"], 100,
                         "Error: Site thresholds wrong")

        self.assertEqual(site2Info["total_running_slots"], 200,
                         "Error: Site thresholds wrong")

        self.assertEqual(site2Info["total_running_jobs"], 0,
                         "Error: Site thresholds wrong")

        self.assertEqual(site2Info["total_pending_jobs"], 0,
                         "Error: Site thresholds wrong")

        self.assertEqual(procThreshold2["task_running_jobs"], 0,
                         "Error: Site thresholds wrong")

        self.assertEqual(procThreshold2["task_pending_jobs"], 0,
                         "Error: Site thresholds wrong")

        self.assertEqual(procThreshold2["max_slots"], 50,
                         "Error: Site thresholds wrong")

        self.assertEqual(procThreshold2["pending_slots"], 30,
                         "Error: Site thresholds wrong")

        self.assertEqual(mergeThreshold2["task_running_jobs"], 0,
                         "Error: Site thresholds wrong")

        self.assertEqual(mergeThreshold2["task_pending_jobs"], 0,
                         "Error: Site thresholds wrong")

        self.assertEqual(mergeThreshold2["max_slots"], 135,
                         "Error: Site thresholds wrong")

        self.assertEqual(mergeThreshold2["pending_slots"], 100,
                         "Error: Site thresholds wrong")

    def testList(self):
        """
        _testList_

        Test the functions that list thresholds for creating jobs and submitting
        jobs.
        """
        myResourceControl = ResourceControl()
        myResourceControl.insertSite("testSite1", 10, 20, "testSE1", "testCE1", "T1_US_FNAL", "LsfPlugin")
        myResourceControl.insertSite("testSite2", 20, 40, "testSE2", "testCE2", "T3_US_FNAL", "LsfPlugin")

        myResourceControl.insertThreshold("testSite1", "Processing", 20, 10)
        myResourceControl.insertThreshold("testSite1", "Merge", 200, 100)
        myResourceControl.insertThreshold("testSite2", "Processing", 50, 25)
        myResourceControl.insertThreshold("testSite2", "Merge", 135, 65)

        self.createJobs()

        createThresholds = myResourceControl.listThresholdsForCreate()
        submitThresholds = myResourceControl.listThresholdsForSubmit()

        self.assertEqual(len(createThresholds), 2,
                         "Error: Wrong number of sites in create thresholds")

        self.assertEqual(createThresholds["testSite1"]["total_slots"], 10,
                         "Error: Wrong number of slots for site 1")

        self.assertEqual(createThresholds["testSite2"]["total_slots"], 20,
                         "Error: Wrong number of slots for site 2")

        # We should have two running jobs with locations at site one,
        # two running jobs without locations at site two, and one running
        # job without a location at site one and two.
        self.assertEqual(createThresholds["testSite1"]["pending_jobs"], {0: 4},
                         "Error: Wrong number of pending jobs for site 1")

        # We should have one running job with a location at site 2 and
        # another running job without a location.
        self.assertEqual(createThresholds["testSite2"]["pending_jobs"], {0: 2},
                         "Error: Wrong number of pending jobs for site 2")

        # We should also have a phedex_name
        self.assertEqual(createThresholds["testSite1"]["cms_name"], "T1_US_FNAL")
        self.assertEqual(createThresholds["testSite2"]["cms_name"], "T3_US_FNAL")

        mergeThreshold1 = None
        mergeThreshold2 = None
        procThreshold1 = None
        procThreshold2 = None
        self.assertEqual(set(submitThresholds.keys()), set(["testSite1", "testSite2"]))
        for taskType, threshold in viewitems(submitThresholds["testSite1"]["thresholds"]):
            if taskType == "Merge":
                mergeThreshold1 = threshold
            elif taskType == "Processing":
                procThreshold1 = threshold
        for taskType, threshold in viewitems(submitThresholds["testSite2"]["thresholds"]):
            if taskType == "Merge":
                mergeThreshold2 = threshold
            elif taskType == "Processing":
                procThreshold2 = threshold

        self.assertEqual(submitThresholds["testSite1"]["total_running_jobs"], 0,
                         "Error: Wrong number of running jobs for submit thresholds.")
        self.assertEqual(submitThresholds["testSite2"]["total_running_jobs"], 1,
                         "Error: Wrong number of running jobs for submit thresholds.")
        self.assertEqual(submitThresholds["testSite1"]["total_pending_jobs"], 1,
                         "Error: Wrong number of pending jobs for submit thresholds.")
        self.assertEqual(submitThresholds["testSite2"]["total_pending_jobs"], 0,
                         "Error: Wrong number of pending jobs for submit thresholds.")

        self.assertEqual(mergeThreshold1["task_running_jobs"], 0,
                         "Error: Wrong number of task running jobs for submit thresholds.")
        self.assertEqual(mergeThreshold1["task_pending_jobs"], 0,
                         "Error: Wrong number of task running jobs for submit thresholds.")
        self.assertEqual(procThreshold1["task_running_jobs"], 0,
                         "Error: Wrong number of task running jobs for submit thresholds.")
        self.assertEqual(procThreshold1["task_pending_jobs"], 1,
                         "Error: Wrong number of task running jobs for submit thresholds.")
        self.assertEqual(mergeThreshold2["task_running_jobs"], 0,
                         "Error: Wrong number of task running jobs for submit thresholds.")
        self.assertEqual(mergeThreshold2["task_pending_jobs"], 0,
                         "Error: Wrong number of task running jobs for submit thresholds.")
        self.assertEqual(procThreshold2["task_running_jobs"], 1,
                         "Error: Wrong number of task running jobs for submit thresholds.")
        self.assertEqual(procThreshold2["task_pending_jobs"], 0,
                         "Error: Wrong number of task running jobs for submit thresholds.")

        return

    def testListSiteInfo(self):
        """
        _testListSiteInfo_

        Verify that the listSiteInfo() methods works properly.
        """
        myResourceControl = ResourceControl()
        myResourceControl.insertSite("testSite1", 10, 20, "testSE1", "testCE1")
        myResourceControl.insertSite("testSite2", 100, 200, "testSE2", "testCE2")

        siteInfo = myResourceControl.listSiteInfo("testSite1")

        self.assertEqual(siteInfo["site_name"], "testSite1",
                         "Error: Site name is wrong.")

        self.assertEqual(siteInfo["pnn"], ["testSE1"],
                         "Error: SE name is wrong.")

        self.assertEqual(siteInfo["ce_name"], "testCE1",
                         "Error: CE name is wrong.")

        self.assertEqual(siteInfo["pending_slots"], 10,
                         "Error: Pending slots is wrong.")

        self.assertEqual(siteInfo["running_slots"], 20,
                         "Error: Pending slots is wrong.")

        return

    def testUpdateJobSlots(self):
        """
        _testUpdateJobSlots_

        Verify that it is possible to update the number of job slots at a site.
        """
        myResourceControl = ResourceControl()
        myResourceControl.insertSite("testSite1", 10, 20, "testSE1", "testCE1")

        siteInfo = myResourceControl.listSiteInfo("testSite1")

        self.assertEqual(siteInfo["pending_slots"], 10, "Error: Pending slots is wrong.")
        self.assertEqual(siteInfo["running_slots"], 20, "Error: Running slots is wrong.")

        myResourceControl.setJobSlotsForSite("testSite1", pendingJobSlots=20)

        siteInfo = myResourceControl.listSiteInfo("testSite1")

        self.assertEqual(siteInfo["pending_slots"], 20, "Error: Pending slots is wrong.")

        myResourceControl.setJobSlotsForSite("testSite1", runningJobSlots=40)

        siteInfo = myResourceControl.listSiteInfo("testSite1")

        self.assertEqual(siteInfo["running_slots"], 40, "Error: Running slots is wrong.")

        myResourceControl.setJobSlotsForSite("testSite1", 5, 10)

        siteInfo = myResourceControl.listSiteInfo("testSite1")

        self.assertEqual(siteInfo["pending_slots"], 5, "Error: Pending slots is wrong.")
        self.assertEqual(siteInfo["running_slots"], 10, "Error: Running slots is wrong.")

        return

    def testThresholdsForSite(self):
        """
        _testThresholdsForSite_

        Check that we can get the thresholds in intelligible form
        for each site
        """

        myResourceControl = ResourceControl()
        myResourceControl.insertSite("testSite1", 20, 40, "testSE1", "testCE1")
        myResourceControl.insertThreshold("testSite1", "Processing", 10, 8)
        myResourceControl.insertThreshold("testSite1", "Merge", 5, 3)

        result = myResourceControl.thresholdBySite(siteName="testSite1")
        procInfo = {}
        mergInfo = {}
        for res in result:
            if res['task_type'] == 'Processing':
                procInfo = res
            elif res['task_type'] == 'Merge':
                mergInfo = res
        self.assertEqual(procInfo.get('pending_slots', None), 20)
        self.assertEqual(procInfo.get('running_slots', None), 40)
        self.assertEqual(procInfo.get('max_slots', None), 10)
        self.assertEqual(procInfo.get('task_pending_slots', None), 8)
        self.assertEqual(mergInfo.get('pending_slots', None), 20)
        self.assertEqual(mergInfo.get('running_slots', None), 40)
        self.assertEqual(mergInfo.get('max_slots', None), 5)
        self.assertEqual(mergInfo.get('task_pending_slots', None), 3)

        return

    def testThresholdPriority(self):
        """
        _testThresholdPriority_

        Test that we get things back in priority order
        """

        myResourceControl = ResourceControl()
        myResourceControl.insertSite("testSite1", 20, 40, "testSE1", "testCE1")
        myResourceControl.insertThreshold("testSite1", "Processing", 10, 8)
        myResourceControl.insertThreshold("testSite1", "Merge", 5, 3)

        # test default task priorities
        result = myResourceControl.listThresholdsForSubmit()
        self.assertEqual(result['testSite1']['thresholds']['Merge']['priority'], 4)
        self.assertEqual(result['testSite1']['thresholds']['Processing']['priority'], 0)

        myResourceControl.changeTaskPriority("Merge", 3)
        myResourceControl.changeTaskPriority("Processing", 1)

        result = myResourceControl.listThresholdsForSubmit()
        self.assertEqual(result['testSite1']['thresholds']['Merge']['priority'], 3)
        self.assertEqual(result['testSite1']['thresholds']['Processing']['priority'], 1)

        myResourceControl.changeTaskPriority("Merge", 1)
        myResourceControl.changeTaskPriority("Processing", 3)

        result = myResourceControl.listThresholdsForSubmit()
        self.assertEqual(result['testSite1']['thresholds']['Merge']['priority'], 1)
        self.assertEqual(result['testSite1']['thresholds']['Processing']['priority'], 3)

        return

    def testChangeSiteState(self):
        """
        _testNewState_

        Check that we can change the state between different values and
        retrieve it through the threshold methods
        """
        self.tempDir = self.testInit.generateWorkDir()
        config = self.createConfig()
        myResourceControl = ResourceControl(config)
        myResourceControl.insertSite("testSite1", 20, 40, "testSE1", "testCE1")
        myResourceControl.insertThreshold("testSite1", "Processing", 10, 5)

        result = myResourceControl.listThresholdsForCreate()
        self.assertEqual(result['testSite1']['state'], 'Normal', 'Error: Wrong site state')

        myResourceControl.changeSiteState("testSite1", "Down")
        result = myResourceControl.listThresholdsForCreate()
        self.assertEqual(result['testSite1']['state'], 'Down', 'Error: Wrong site state')

        # If you set the value to 'Normal' instead of 'Down' this test should FAIL
        # self.assertEqual(result['testSite1']['state'], 'Normal', 'Error: Wrong site state')

    def testAbortedState(self):
        """
        _testAbortedState_

        Check that we can kill jobs when a site is set to aborted
        ### We no longer need this test as we are not killing jobs that are running
        """
        self.tempDir = self.testInit.generateWorkDir()
        config = self.createConfig()
        myResourceControl = ResourceControl(config)
        myResourceControl.insertSite("testSite1", 10, 20, "testSE1", "testCE1", "T1_US_FNAL", "MockPlugin")
        myResourceControl.insertSite("testSite2", 20, 40, "testSE2", "testCE2", "T1_IT_CNAF", "MockPlugin")

        myResourceControl.insertThreshold("testSite1", "Processing", 20, 10)
        myResourceControl.insertThreshold("testSite1", "Merge", 200, 100)
        myResourceControl.insertThreshold("testSite2", "Processing", 50, 25)
        myResourceControl.insertThreshold("testSite2", "Merge", 135, 65)

        self.createJobs()

        myResourceControl.changeSiteState("testSite1", "Aborted")

        ## Now check the tempDir for a FWJR for the killed job
        reportPath = os.path.join(self.tempDir, "Report.0.pkl")
        report = Report()
        report.load(reportPath)
        self.assertEqual(report.getExitCode(), 71301)
        return

    def createConfig(self):
        """
        _createConfig_

        Create a config and save it to the temp dir.  Set the WMAGENT_CONFIG
        environment variable so the config gets picked up.
        """
        config = Configuration()
        config.section_("General")
        config.General.workDir = os.getenv("TESTDIR", os.getcwd())
        config.section_("Agent")
        config.Agent.componentName = "resource_control_t"
        config.section_("CoreDatabase")
        config.CoreDatabase.connectUrl = os.getenv("DATABASE")
        config.CoreDatabase.socket = os.getenv("DBSOCK")
        config.section_("JobStateMachine")
        config.JobStateMachine.couchurl = os.getenv('COUCHURL')
        config.JobStateMachine.couchDBName = "bossair_t"
        config.JobStateMachine.jobSummaryDBName = 'wmagent_summary_t'
        config.JobStateMachine.summaryStatsDBName = 'stat_summary_t'
        config.section_("BossAir")
        config.BossAir.pluginDir = "WMCore.BossAir.Plugins"
        config.BossAir.pluginNames = ["MockPlugin"]
        config.BossAir.section_("MockPlugin")
        config.BossAir.MockPlugin.fakeReport = os.path.join(getTestBase(),
                                                            'WMComponent_t/JobAccountant_t/fwjrs',
                                                            "MergeSuccess.pkl")

        configHandle = open(os.path.join(self.tempDir, "config.py"), "w")
        configHandle.write(str(config))
        configHandle.close()

        os.environ["WMAGENT_CONFIG"] = os.path.join(self.tempDir, "config.py")
        return config

    def testInsertSite(self):
        """
        _testInsertSite_

        Test to see if we can insert a fake test site alone
        with a single option
        """
        self.createConfig()

        resControlPath = os.path.join(getTestBase(), "../../bin/wmagent-resource-control")
        env = os.environ
        env['PYTHONPATH'] = ":".join(sys.path)
        if PY3:
            cmdline = ["python3", resControlPath, "--add-Test"]
        else:
            cmdline = [resControlPath, "--add-Test"]
        retval = subprocess.Popen(cmdline,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.STDOUT,
                                  env=env)
        (_, _) = retval.communicate()
        myResourceControl = ResourceControl()
        result = myResourceControl.listThresholdsForSubmit()
        self.assertEqual(len(result), 1)
        self.assertTrue('CERN' in result)
        for x in result:
            self.assertEqual(len(result[x]['thresholds']), 7)
            self.assertEqual(result[x]['total_pending_slots'], 500)
            self.assertEqual(result[x]['total_running_slots'], 1)
            for taskType, thresh in viewitems(result[x]['thresholds']):
                if taskType == 'Processing':
                    self.assertEqual(thresh['priority'], 0)
                    self.assertEqual(thresh['max_slots'], 1)

        # Verify that sites with more than one SE were added correctly.
        cernInfo = myResourceControl.listSiteInfo("CERN")
        self.assertTrue(len(cernInfo["pnn"]) == 2)
        return


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# pylint: disable-msg=C0103
"""
Unittests for Utilities functions
"""

from __future__ import division, print_function
import time
import threading
import unittest
from Utils.Throttled import UserThrottle, UserThrottleTime
from cherrypy import HTTPError



class UtilitiesTests(unittest.TestCase):
    """
    unittest for Utilities functions
    """
    def setUp(self):
        self.limit = 100
        self.thr = UserThrottleTime(limit=self.limit)
        self.thrc = UserThrottle(limit=3)
        self.thr3 = UserThrottleTime(limit=3)
        self.thr10 = UserThrottleTime(limit=10)

    def testthrottledTimes(self):
        """
        Test that throttled function behave slower than normal
        """
        @self.thr.make_throttled()
        def throttled():
            "Test function for throttled"
            return time.time()
        thr_times = []
        xxx_times = []
        for _ in range(self.limit-1):
            thr_times.append(throttled())
        for _ in range(self.limit-1):
            xxx_times.append(time.time())
        thr_range = max(thr_times) - min(thr_times)
        nor_range = max(xxx_times) - min(xxx_times)
        self.assertEqual(thr_range > nor_range, True)
        self.assertEqual((thr_range - nor_range) > nor_range, True)

    def testthrottledTimeRange(self):
        """
        Test that throttled within certain time range
        """

        @self.thr10.make_throttled()
        def throttled():
            "Test function for throttled"
            return time.time()

        @self.thr3.make_throttled(trange=2)
        def throttled2():
            "Test function for throttled"
            return time.time()

        def test(dim):
            for _ in range(dim):
                throttled2()
        self.assertRaises(HTTPError, test, 5)
        # once assertion happens user will be allowed to make N-additional
        # calls based on elapsed time of throttler
        time.sleep(1)
        throttled2()

        # here we should be fine because throttled function decorated with
        # limit=10
        for _ in range(5):
            throttled()

        # sleep 2 sec (our trange) and use again our throttled2 function
        # this time it should be fine
        time.sleep(2)
        throttled2()
        # but we we'll call it multiple times we'll get back exception
        self.assertRaises(HTTPError, test, 5)

    def testthrottledCounter(self):
        """
        Test that throttled counter version
        """

        @self.thrc.make_throttled()
        def throttled():
            "Test function for throttled"
            return time.time()

        def test(dim):
            for _ in range(dim):
                throttled()
            self.assertRaises(HTTPError, testCounter, 5)

        def testCounter(iterations):
            threads = []
            for _ in range(iterations):
                thr = threading.Thread(target=test, args=(50,))
                thr.start()
                threads.append(thr)
            for thr in threads:
                thr.join()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

from __future__ import division, print_function

import json
import os
import unittest

from Utils.PythonVersion import PY3
from WMCore.WMStats.DataStructs.DataCache import DataCache


class DataCacheTests(unittest.TestCase):
    """
    Unit tests for WMStats DataCache
    """

    def setUp(self):
        self.fileCache = os.path.join(os.path.dirname(__file__), 'DataCache.json')
        with open(self.fileCache) as jo:
            data = json.load(jo)
        DataCache().setlatestJobData(data)
        if PY3:
            self.assertItemsEqual = self.assertCountEqual

    def tearDown(self):
        pass

    def testDuration(self):
        self.assertEqual(300, DataCache.getDuration())

        DataCache.setDuration(100)
        self.assertEqual(100, DataCache.getDuration())

    def testLatestJobData(self):
        self.assertEqual(20, len(DataCache.getlatestJobData()))
        self.assertItemsEqual(['time', 'data'], list(DataCache._lastedActiveDataFromAgent))

        DataCache.setlatestJobData("ALAN")
        self.assertEqual("ALAN", DataCache.getlatestJobData())
        self.assertItemsEqual(['time', 'data'], list(DataCache._lastedActiveDataFromAgent))

    def testLatestJobDataExpired(self):
        self.assertFalse(DataCache.islatestJobDataExpired())

        DataCache.setDuration(-1)
        self.assertTrue(DataCache.islatestJobDataExpired())

        DataCache.setDuration(300)
        self.assertFalse(DataCache.islatestJobDataExpired())
        DataCache._lastedActiveDataFromAgent = {}
        self.assertTrue(DataCache.islatestJobDataExpired())

        self.assertEqual({}, DataCache.getlatestJobData())

    def testFilterData(self):
        data = list(DataCache.filterData(filterDict={}, maskList=['RequestType']))
        self.assertEqual(20, len(data))
        self.assertItemsEqual(['ReReco', 'MonteCarlo', 'StepChain', 'MonteCarloFromGEN',
                               'ReDigi', 'TaskChain', 'DQMHarvest'], set(list(data)))

        data = list(DataCache.filterData(filterDict={}, maskList=['Campaign', 'RequestType']))
        self.assertEqual(40, len(data))

        data = list(DataCache.filterData(filterDict={'IncludeParents': 'True'}, maskList=['Campaign']))
        self.assertEqual(2, len(data))

        data = list(DataCache.filterData(filterDict={'Campaign': 'CMSSW_9_4_0__test2inwf-1510737328'},
                                         maskList=['RequestName']))
        self.assertItemsEqual(["amaltaro_TaskChain_InclParents_HG1812_Validation_181203_121005_1483"],
                              data)

    def testFilterDataByRequest(self):
        data = list(DataCache.filterDataByRequest(filterDict={}, maskList='RequestType'))
        self.assertEqual(20, len(data))
        self.assertItemsEqual(['RequestName', 'RequestType'], list(data[0]))
        reqTypes = [item['RequestType'] for item in data]
        self.assertItemsEqual(['ReReco', 'MonteCarlo', 'StepChain', 'MonteCarloFromGEN',
                               'ReDigi', 'TaskChain', 'DQMHarvest'], set(list(reqTypes)))

        data = list(DataCache.filterDataByRequest(filterDict={}, maskList=['Campaign', 'RequestType']))
        self.assertEqual(20, len(data))

        data = list(DataCache.filterDataByRequest(filterDict={'IncludeParents': 'True'}, maskList=['Campaign']))
        self.assertEqual(2, len(data))

        data = list(DataCache.filterDataByRequest(filterDict={'Campaign': 'CMSSW_9_4_0__test2inwf-1510737328'},
                                                  maskList=['RequestName']))
        self.assertEqual(1, len(data))
        self.assertEqual("amaltaro_TaskChain_InclParents_HG1812_Validation_181203_121005_1483",
                         data[0]['RequestName'])


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_Express_t_

Unit tests for the Express workflow.
"""

from __future__ import division, print_function
from future.utils import viewitems

import threading
import unittest
from copy import deepcopy

from Utils.PythonVersion import PY3

from WMCore.DAOFactory import DAOFactory
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow import Workflow
from WMCore.WMSpec.StdSpecs.Express import ExpressWorkloadFactory
from WMCore.WorkQueue.WMBSHelper import WMBSHelper
from WMQuality.TestInitCouchApp import TestInitCouchApp

REQUEST = {
    "AcquisitionEra": "TestAcquisitionEra",
    "AlcaHarvestTimeout": 12 * 3600,
    "AlcaHarvestCondLFNBase": "somelfn",
    "AlcaHarvestLumiURL": "someurl",
    "AlcaSkims": ["TkAlMinBias", "PromptCalibProd"],
    "CMSSWVersion": "CMSSW_9_0_0",
    "DQMSequences": ["@common"],
    "DQMUploadProxy": "/somepath/proxy",
    "DQMUploadUrl": "https://cmsweb.cern.ch/dqm/offline",
    "GlobalTag": "SomeGlobalTag",
    "GlobalTagTransaction": "Express_123456",
    "GlobalTagConnect": "frontier://PromptProd/CMS_CONDITIONS",
    "MaxInputRate": 23 * 1000,
    "MaxInputEvents": 400,
    "MaxInputSize": 2 * 1024 * 1024 * 1024,
    "MaxInputFiles": 15,
    "MaxLatency": 15 * 23,
    "Outputs": [{'dataTier': "FEVT", 'eventContent': "FEVT",
                 'selectEvents': ["Path1:HLT,Path2:HLT"], 'primaryDataset': "PrimaryDataset1"},
                {'dataTier': "ALCARECO", 'eventContent': "ALCARECO", 'primaryDataset': "StreamExpress"},
                {'dataTier': "DQMIO", 'eventContent': "DQMIO", 'primaryDataset': "StreamExpress"}],
    "PeriodicHarvestInterval": 20 * 60,
    "ProcessingString": "Express",
    "ProcessingVersion": 9,
    "RecoCMSSWVersion": "CMSSW_9_0_1",
    "RecoScramArch": "slc6_amd64_gcc630",
    "RunNumber": 123456,
    "ScramArch": "slc6_amd64_gcc530",
    "Scenario": "test_scenario",
    "SpecialDataset": "StreamExpress",
    "StreamName": "Express"
}


class ExpressTest(unittest.TestCase):
    def setUp(self):
        """
        _setUp_

        Initialize the database and couch.
        """
        self.testInit = TestInitCouchApp(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules=["WMCore.WMBS"],
                                useDefault=False)
        self.testDir = self.testInit.generateWorkDir()

        myThread = threading.currentThread()
        self.daoFactory = DAOFactory(package="WMCore.WMBS",
                                     logger=myThread.logger,
                                     dbinterface=myThread.dbi)
        self.listTasksByWorkflow = self.daoFactory(classname="Workflow.LoadFromName")
        self.listFilesets = self.daoFactory(classname="Fileset.List")
        self.listSubsMapping = self.daoFactory(classname="Subscriptions.ListSubsAndFilesetsFromWorkflow")
        if PY3:
            self.assertItemsEqual = self.assertCountEqual
        return

    def tearDown(self):
        """
        _tearDown_

        Clear out the database.
        """
        self.testInit.clearDatabase()
        self.testInit.delWorkDir()
        return

    def testExpress(self):
        """
        _testExpress_

        Create an Express workflow
        and verify it installs into WMBS correctly.
        """
        testArguments = ExpressWorkloadFactory.getTestArguments()
        testArguments.update(deepcopy(REQUEST))
        testArguments['RecoCMSSWVersion'] = "CMSSW_9_0_0"

        factory = ExpressWorkloadFactory()
        testWorkload = factory.factoryWorkloadConstruction("TestWorkload", testArguments)
        testWorkload.setSpecUrl("somespec")
        testWorkload.setOwnerDetails("Dirk.Hufnagel@cern.ch", "T0")

        testWMBSHelper = WMBSHelper(testWorkload, "Express", cachepath=self.testDir)
        testWMBSHelper.createTopLevelFileset()
        testWMBSHelper._createSubscriptionsInWMBS(testWMBSHelper.topLevelTask, testWMBSHelper.topLevelFileset)

        expressWorkflow = Workflow(name="TestWorkload",
                                   task="/TestWorkload/Express")
        expressWorkflow.load()
        self.assertEqual(len(expressWorkflow.outputMap), len(testArguments["Outputs"]) + 1,
                         "Error: Wrong number of WF outputs in the Express WF.")

        goldenOutputMods = {"write_PrimaryDataset1_FEVT": "FEVT",
                            "write_StreamExpress_ALCARECO": "ALCARECO",
                            "write_StreamExpress_DQMIO": "DQMIO"}
        for goldenOutputMod, tier in viewitems(goldenOutputMods):
            fset = goldenOutputMod + tier
            mergedOutput = expressWorkflow.outputMap[fset][0]["merged_output_fileset"]
            unmergedOutput = expressWorkflow.outputMap[fset][0]["output_fileset"]
            mergedOutput.loadData()
            unmergedOutput.loadData()

            if goldenOutputMod != "write_StreamExpress_ALCARECO":
                self.assertEqual(mergedOutput.name,
                                 "/TestWorkload/Express/ExpressMerge%s/merged-Merged%s" % (goldenOutputMod, tier),
                                 "Error: Merged output fileset is wrong: %s" % mergedOutput.name)
            self.assertEqual(unmergedOutput.name, "/TestWorkload/Express/unmerged-%s" % fset,
                             "Error: Unmerged output fileset is wrong: %s" % unmergedOutput.name)

        logArchOutput = expressWorkflow.outputMap["logArchive"][0]["merged_output_fileset"]
        unmergedLogArchOutput = expressWorkflow.outputMap["logArchive"][0]["output_fileset"]
        logArchOutput.loadData()
        unmergedLogArchOutput.loadData()

        self.assertEqual(logArchOutput.name, "/TestWorkload/Express/unmerged-logArchive",
                         "Error: LogArchive output fileset is wrong.")
        self.assertEqual(unmergedLogArchOutput.name, "/TestWorkload/Express/unmerged-logArchive",
                         "Error: LogArchive output fileset is wrong.")

        alcaSkimWorkflow = Workflow(name="TestWorkload",
                                    task="/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO")
        alcaSkimWorkflow.load()
        self.assertEqual(len(alcaSkimWorkflow.outputMap), len(testArguments["AlcaSkims"]) + 1,
                         "Error: Wrong number of WF outputs in the AlcaSkim WF.")

        goldenOutputMods = {"ALCARECOStreamPromptCalibProd": "ALCAPROMPT",
                            "ALCARECOStreamTkAlMinBias": "ALCARECO"}
        for goldenOutputMod, tier in viewitems(goldenOutputMods):
            fset = goldenOutputMod + tier
            mergedOutput = alcaSkimWorkflow.outputMap[fset][0]["merged_output_fileset"]
            unmergedOutput = alcaSkimWorkflow.outputMap[fset][0]["output_fileset"]
            mergedOutput.loadData()
            unmergedOutput.loadData()
            self.assertEqual(mergedOutput.name,
                             "/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/unmerged-%s" % fset,
                             "Error: Merged output fileset is wrong: %s" % mergedOutput.name)
            self.assertEqual(unmergedOutput.name,
                             "/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/unmerged-%s" % fset,
                             "Error: Unmerged output fileset is wrong: %s" % unmergedOutput.name)

        logArchOutput = alcaSkimWorkflow.outputMap["logArchive"][0]["merged_output_fileset"]
        unmergedLogArchOutput = alcaSkimWorkflow.outputMap["logArchive"][0]["output_fileset"]
        logArchOutput.loadData()
        unmergedLogArchOutput.loadData()

        self.assertEqual(logArchOutput.name,
                         "/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/unmerged-logArchive",
                         "Error: LogArchive output fileset is wrong.")
        self.assertEqual(unmergedLogArchOutput.name,
                         "/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/unmerged-logArchive",
                         "Error: LogArchive output fileset is wrong.")

        for dqmString in ["ExpressMergewrite_StreamExpress_DQMIOEndOfRunDQMHarvestMerged",
                          "ExpressMergewrite_StreamExpress_DQMIOPeriodicDQMHarvestMerged"]:
            dqmTask = "/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/%s" % dqmString

            dqmWorkflow = Workflow(name="TestWorkload",
                                   task=dqmTask)
            dqmWorkflow.load()

            logArchOutput = dqmWorkflow.outputMap["logArchive"][0]["merged_output_fileset"]
            unmergedLogArchOutput = dqmWorkflow.outputMap["logArchive"][0]["output_fileset"]
            logArchOutput.loadData()
            unmergedLogArchOutput.loadData()

            self.assertEqual(logArchOutput.name,
                             "%s/unmerged-logArchive" % dqmTask,
                             "Error: LogArchive output fileset is wrong.")
            self.assertEqual(unmergedLogArchOutput.name,
                             "%s/unmerged-logArchive" % dqmTask,
                             "Error: LogArchive output fileset is wrong.")

        goldenOutputMods = {"write_PrimaryDataset1_FEVT": "FEVT",
                            "write_StreamExpress_DQMIO": "DQMIO"}
        for goldenOutputMod, tier in viewitems(goldenOutputMods):
            mergeWorkflow = Workflow(name="TestWorkload",
                                     task="/TestWorkload/Express/ExpressMerge%s" % goldenOutputMod)
            mergeWorkflow.load()

            self.assertEqual(len(mergeWorkflow.outputMap), 2,
                             "Error: Wrong number of WF outputs.")

            mergedMergeOutput = mergeWorkflow.outputMap["Merged%s" % tier][0]["merged_output_fileset"]
            unmergedMergeOutput = mergeWorkflow.outputMap["Merged%s" % tier][0]["output_fileset"]

            mergedMergeOutput.loadData()
            unmergedMergeOutput.loadData()

            self.assertEqual(mergedMergeOutput.name,
                             "/TestWorkload/Express/ExpressMerge%s/merged-Merged%s" % (goldenOutputMod, tier),
                             "Error: Merged output fileset is wrong.")
            self.assertEqual(unmergedMergeOutput.name,
                             "/TestWorkload/Express/ExpressMerge%s/merged-Merged%s" % (goldenOutputMod, tier),
                             "Error: Unmerged output fileset is wrong.")

            logArchOutput = mergeWorkflow.outputMap["logArchive"][0]["merged_output_fileset"]
            unmergedLogArchOutput = mergeWorkflow.outputMap["logArchive"][0]["output_fileset"]
            logArchOutput.loadData()
            unmergedLogArchOutput.loadData()

            self.assertEqual(logArchOutput.name,
                             "/TestWorkload/Express/ExpressMerge%s/merged-logArchive" % goldenOutputMod,
                             "Error: LogArchive output fileset is wrong: %s" % logArchOutput.name)
            self.assertEqual(unmergedLogArchOutput.name,
                             "/TestWorkload/Express/ExpressMerge%s/merged-logArchive" % goldenOutputMod,
                             "Error: LogArchive output fileset is wrong.")

        topLevelFileset = Fileset(name="TestWorkload-Express")
        topLevelFileset.loadData()

        expressSubscription = Subscription(fileset=topLevelFileset, workflow=expressWorkflow)
        expressSubscription.loadData()

        self.assertEqual(expressSubscription["type"], "Express",
                         "Error: Wrong subscription type.")
        self.assertEqual(expressSubscription["split_algo"], "Express",
                         "Error: Wrong split algorithm. %s" % expressSubscription["split_algo"])

        alcaRecoFileset = Fileset(name="/TestWorkload/Express/unmerged-write_StreamExpress_ALCARECOALCARECO")
        alcaRecoFileset.loadData()

        alcaSkimSubscription = Subscription(fileset=alcaRecoFileset, workflow=alcaSkimWorkflow)
        alcaSkimSubscription.loadData()

        self.assertEqual(alcaSkimSubscription["type"], "Express",
                         "Error: Wrong subscription type.")
        self.assertEqual(alcaSkimSubscription["split_algo"], "ExpressMerge",
                         "Error: Wrong split algorithm. %s" % alcaSkimSubscription["split_algo"])

        mergedDQMFileset = Fileset(
            name="/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/merged-MergedDQMIO")
        mergedDQMFileset.loadData()

        dqmSubscription = Subscription(fileset=mergedDQMFileset, workflow=dqmWorkflow)
        dqmSubscription.loadData()

        self.assertEqual(dqmSubscription["type"], "Harvesting",
                         "Error: Wrong subscription type.")
        self.assertEqual(dqmSubscription["split_algo"], "Harvest",
                         "Error: Wrong split algo.")

        unmergedOutputs = {"write_PrimaryDataset1_FEVT": "FEVT",
                           "write_StreamExpress_DQMIO": "DQMIO"}
        for unmergedOutput, tier in viewitems(unmergedOutputs):
            fset = unmergedOutput + tier
            unmergedDataTier = Fileset(name="/TestWorkload/Express/unmerged-%s" % fset)
            unmergedDataTier.loadData()
            dataTierMergeWorkflow = Workflow(name="TestWorkload",
                                             task="/TestWorkload/Express/ExpressMerge%s" % unmergedOutput)
            dataTierMergeWorkflow.load()
            mergeSubscription = Subscription(fileset=unmergedDataTier, workflow=dataTierMergeWorkflow)
            mergeSubscription.loadData()

            self.assertEqual(mergeSubscription["type"], "Merge",
                             "Error: Wrong subscription type.")
            self.assertEqual(mergeSubscription["split_algo"], "ExpressMerge",
                             "Error: Wrong split algorithm. %s" % mergeSubscription["split_algo"])

        goldenOutputMods = {"write_PrimaryDataset1_FEVT": "FEVT",
                            "write_StreamExpress_ALCARECO": "ALCARECO",
                            "write_StreamExpress_DQMIO": "DQMIO"}
        for goldenOutputMod, tier in viewitems(goldenOutputMods):
            fset = goldenOutputMod + tier
            unmergedFileset = Fileset(name="/TestWorkload/Express/unmerged-%s" % fset)
            unmergedFileset.loadData()
            cleanupWorkflow = Workflow(name="TestWorkload",
                                       task="/TestWorkload/Express/ExpressCleanupUnmerged%s" % goldenOutputMod)
            cleanupWorkflow.load()
            cleanupSubscription = Subscription(fileset=unmergedFileset, workflow=cleanupWorkflow)
            cleanupSubscription.loadData()

            self.assertEqual(cleanupSubscription["type"], "Cleanup",
                             "Error: Wrong subscription type.")
            self.assertEqual(cleanupSubscription["split_algo"], "SiblingProcessingBased",
                             "Error: Wrong subscription type.")

        expressLogCollect = Fileset(name="/TestWorkload/Express/unmerged-logArchive")
        expressLogCollect.loadData()
        expressLogCollectWorkflow = Workflow(name="TestWorkload",
                                             task="/TestWorkload/Express/ExpressLogCollect")
        expressLogCollectWorkflow.load()
        logCollectSub = Subscription(fileset=expressLogCollect, workflow=expressLogCollectWorkflow)
        logCollectSub.loadData()

        self.assertEqual(logCollectSub["type"], "LogCollect",
                         "Error: Wrong subscription type.")
        self.assertEqual(logCollectSub["split_algo"], "MinFileBased",
                         "Error: Wrong split algorithm.")

        alcaSkimLogCollect = Fileset(
            name="/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/unmerged-logArchive")
        alcaSkimLogCollect.loadData()
        alcaSkimLogCollectWorkflow = Workflow(name="TestWorkload",
                                              task="/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/AlcaSkimLogCollect")
        alcaSkimLogCollectWorkflow.load()
        logCollectSub = Subscription(fileset=alcaSkimLogCollect, workflow=alcaSkimLogCollectWorkflow)
        logCollectSub.loadData()

        self.assertEqual(logCollectSub["type"], "LogCollect",
                         "Error: Wrong subscription type.")
        self.assertEqual(logCollectSub["split_algo"], "MinFileBased",
                         "Error: Wrong split algorithm.")

        goldenOutputMods = ["write_PrimaryDataset1_FEVT", "write_StreamExpress_DQMIO"]
        for goldenOutputMod in goldenOutputMods:
            expressMergeLogCollect = Fileset(
                name="/TestWorkload/Express/ExpressMerge%s/merged-logArchive" % goldenOutputMod)
            expressMergeLogCollect.loadData()
            expressMergeLogCollectWorkflow = Workflow(name="TestWorkload",
                                                      task="/TestWorkload/Express/ExpressMerge%s/Express%sMergeLogCollect" % (
                                                          goldenOutputMod, goldenOutputMod))
            expressMergeLogCollectWorkflow.load()
            logCollectSubscription = Subscription(fileset=expressMergeLogCollect,
                                                  workflow=expressMergeLogCollectWorkflow)
            logCollectSubscription.loadData()

            self.assertEqual(logCollectSub["type"], "LogCollect",
                             "Error: Wrong subscription type.")
            self.assertEqual(logCollectSub["split_algo"], "MinFileBased",
                             "Error: Wrong split algorithm.")

        for dqmStrings in [("ExpressMergewrite_StreamExpress_DQMIOEndOfRunDQMHarvestMerged",
                            "ExpressMergewrite_StreamExpress_DQMIOMergedEndOfRunDQMHarvestLogCollect"),
                           ("ExpressMergewrite_StreamExpress_DQMIOPeriodicDQMHarvestMerged",
                            "ExpressMergewrite_StreamExpress_DQMIOMergedPeriodicDQMHarvestLogCollect")]:
            dqmFileset = "/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/%s/unmerged-logArchive" % \
                         dqmStrings[0]
            dqmHarvestLogCollect = Fileset(name=dqmFileset)
            dqmHarvestLogCollect.loadData()

            dqmTask = "/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/%s/%s" % dqmStrings
            dqmHarvestLogCollectWorkflow = Workflow(name="TestWorkload", task=dqmTask)
            dqmHarvestLogCollectWorkflow.load()

            logCollectSub = Subscription(fileset=dqmHarvestLogCollect, workflow=dqmHarvestLogCollectWorkflow)
            logCollectSub.loadData()

            self.assertEqual(logCollectSub["type"], "LogCollect",
                             "Error: Wrong subscription type.")
            self.assertEqual(logCollectSub["split_algo"], "MinFileBased",
                             "Error: Wrong split algo.")

        return

    def testMemCoresSettings(self):
        """
        _testMemCoresSettings_

        Make sure the multicore and memory setings are properly propagated to
        all tasks and steps.
        """
        testArguments = ExpressWorkloadFactory.getTestArguments()
        testArguments.update(deepcopy(REQUEST))

        factory = ExpressWorkloadFactory()
        testWorkload = factory.factoryWorkloadConstruction("TestWorkload", testArguments)

        # test default values
        taskPaths = ['/TestWorkload/Express', '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO']
        for task in taskPaths:
            taskObj = testWorkload.getTaskByPath(task)
            for step in ('cmsRun1', 'stageOut1', 'logArch1'):
                stepHelper = taskObj.getStepHelper(step)
                self.assertEqual(stepHelper.getNumberOfCores(), 1)
                self.assertEqual(stepHelper.getNumberOfStreams(), 0)
            # then test Memory requirements
            perfParams = taskObj.jobSplittingParameters()['performance']
            self.assertEqual(perfParams['memoryRequirement'], 2300.0)

        # now test case where args are provided
        # testArguments = ExpressWorkloadFactory.getTestArguments()
        testArguments["Multicore"] = 6
        testArguments["Memory"] = 4600.0
        testArguments["EventStreams"] = 3
        testArguments["Outputs"] = deepcopy(REQUEST['Outputs'])

        factory = ExpressWorkloadFactory()
        testWorkload = factory.factoryWorkloadConstruction("TestWorkload", testArguments)

        for task in taskPaths:
            taskObj = testWorkload.getTaskByPath(task)
            for step in ('cmsRun1', 'stageOut1', 'logArch1'):
                stepHelper = taskObj.getStepHelper(step)
                if task == '/TestWorkload/Express' and step == 'cmsRun1':
                    self.assertEqual(stepHelper.getNumberOfCores(), testArguments["Multicore"])
                    self.assertEqual(stepHelper.getNumberOfStreams(), testArguments["EventStreams"])
                elif step in ('stageOut1', 'logArch1'):
                    self.assertEqual(stepHelper.getNumberOfCores(), 1)
                    self.assertEqual(stepHelper.getNumberOfStreams(), 0)
                else:
                    self.assertEqual(stepHelper.getNumberOfCores(), 1, "%s should be single-core" % task)
                    self.assertEqual(stepHelper.getNumberOfStreams(), 0)
            # then test Memory requirements
            perfParams = taskObj.jobSplittingParameters()['performance']
            self.assertEqual(perfParams['memoryRequirement'], testArguments["Memory"])

        return

    def testWithRepackStep(self):
        """
        _testWithRepackStep_

        Make sure we get an initial repack step if CMSSW versions are mismatched
        """
        testArguments = ExpressWorkloadFactory.getTestArguments()
        testArguments.update(deepcopy(REQUEST))

        factory = ExpressWorkloadFactory()
        testWorkload = factory.factoryWorkloadConstruction("TestWorkload", testArguments)

        taskObj = testWorkload.getTaskByPath('/TestWorkload/Express')

        stepHelper = taskObj.getStepHelper('cmsRun1')
        self.assertEqual(stepHelper.getScenario(), "test_scenario")
        self.assertEqual(stepHelper.getCMSSWVersion(), "CMSSW_9_0_0")
        self.assertEqual(stepHelper.getScramArch(), ["slc6_amd64_gcc530"])

        stepHelper = taskObj.getStepHelper('cmsRun2')
        self.assertEqual(stepHelper.getScenario(), "test_scenario")
        self.assertEqual(stepHelper.getCMSSWVersion(), "CMSSW_9_0_1")
        self.assertEqual(stepHelper.getScramArch(), ["slc6_amd64_gcc630"])

        return

    def testFilesets(self):
        """
        Test filesets created for an Express workflow
        """
        # expected tasks, filesets, subscriptions, etc
        expOutTasks = ['/TestWorkload/Express',
                       '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO',
                       '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/ExpressAlcaSkimwrite_StreamExpress_ALCARECOAlcaHarvestALCARECOStreamPromptCalibProd',
                       '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO',
                       '/TestWorkload/Express/ExpressMergewrite_PrimaryDataset1_FEVT']
        expWfTasks = ['/TestWorkload/Express',
                      '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO',
                      '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/AlcaSkimLogCollect',
                      '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/ExpressAlcaSkimwrite_StreamExpress_ALCARECOAlcaHarvestALCARECOStreamPromptCalibProd',
                      '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/ExpressAlcaSkimwrite_StreamExpress_ALCARECOAlcaHarvestALCARECOStreamPromptCalibProd/ExpressAlcaSkimwrite_StreamExpress_ALCARECOAlcaHarvestALCARECOStreamPromptCalibProdConditionSqlite',
                      '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/ExpressAlcaSkimwrite_StreamExpress_ALCARECOAlcaHarvestALCARECOStreamPromptCalibProd/ExpressAlcaSkimwrite_StreamExpress_ALCARECOALCARECOStreamPromptCalibProdAlcaHarvestLogCollect',
                      '/TestWorkload/Express/ExpressCleanupUnmergedwrite_PrimaryDataset1_FEVT',
                      '/TestWorkload/Express/ExpressCleanupUnmergedwrite_StreamExpress_ALCARECO',
                      '/TestWorkload/Express/ExpressCleanupUnmergedwrite_StreamExpress_DQMIO',
                      '/TestWorkload/Express/ExpressLogCollect',
                      '/TestWorkload/Express/ExpressMergewrite_PrimaryDataset1_FEVT',
                      '/TestWorkload/Express/ExpressMergewrite_PrimaryDataset1_FEVT/Expresswrite_PrimaryDataset1_FEVTMergeLogCollect',
                      '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO',
                      '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/ExpressMergewrite_StreamExpress_DQMIOEndOfRunDQMHarvestMerged',
                      '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/ExpressMergewrite_StreamExpress_DQMIOEndOfRunDQMHarvestMerged/ExpressMergewrite_StreamExpress_DQMIOMergedEndOfRunDQMHarvestLogCollect',
                      '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/ExpressMergewrite_StreamExpress_DQMIOPeriodicDQMHarvestMerged',
                      '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/ExpressMergewrite_StreamExpress_DQMIOPeriodicDQMHarvestMerged/ExpressMergewrite_StreamExpress_DQMIOMergedPeriodicDQMHarvestLogCollect',
                      '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/Expresswrite_StreamExpress_DQMIOMergeLogCollect']
        expFsets = ['TestWorkload-Express-Run123456',
                    '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/unmerged-ALCARECOStreamPromptCalibProdALCAPROMPT',
                    '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/unmerged-ALCARECOStreamTkAlMinBiasALCARECO',
                    '/TestWorkload/Express/unmerged-write_RAWRAW',
                    '/TestWorkload/Express/unmerged-write_StreamExpress_ALCARECOALCARECO',
                    '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/ExpressAlcaSkimwrite_StreamExpress_ALCARECOAlcaHarvestALCARECOStreamPromptCalibProd/unmerged-logArchive',
                    '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/ExpressAlcaSkimwrite_StreamExpress_ALCARECOAlcaHarvestALCARECOStreamPromptCalibProd/unmerged-SqliteALCAPROMPT',
                    '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/unmerged-logArchive',
                    '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/ExpressMergewrite_StreamExpress_DQMIOPeriodicDQMHarvestMerged/unmerged-logArchive',
                    '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/merged-MergedDQMIO',
                    '/TestWorkload/Express/unmerged-write_StreamExpress_DQMIODQMIO',
                    '/TestWorkload/Express/ExpressMergewrite_PrimaryDataset1_FEVT/merged-logArchive',
                    '/TestWorkload/Express/ExpressMergewrite_PrimaryDataset1_FEVT/merged-MergedFEVT',
                    '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/ExpressMergewrite_StreamExpress_DQMIOEndOfRunDQMHarvestMerged/unmerged-logArchive',
                    '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/merged-logArchive',
                    '/TestWorkload/Express/unmerged-logArchive',
                    '/TestWorkload/Express/unmerged-write_PrimaryDataset1_FEVTFEVT']

        subMaps = [(8,
                    '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/ExpressAlcaSkimwrite_StreamExpress_ALCARECOAlcaHarvestALCARECOStreamPromptCalibProd/unmerged-logArchive',
                    '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/ExpressAlcaSkimwrite_StreamExpress_ALCARECOAlcaHarvestALCARECOStreamPromptCalibProd/ExpressAlcaSkimwrite_StreamExpress_ALCARECOALCARECOStreamPromptCalibProdAlcaHarvestLogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (7,
                    '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/ExpressAlcaSkimwrite_StreamExpress_ALCARECOAlcaHarvestALCARECOStreamPromptCalibProd/unmerged-SqliteALCAPROMPT',
                    '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/ExpressAlcaSkimwrite_StreamExpress_ALCARECOAlcaHarvestALCARECOStreamPromptCalibProd/ExpressAlcaSkimwrite_StreamExpress_ALCARECOAlcaHarvestALCARECOStreamPromptCalibProdConditionSqlite',
                    'Condition',
                    'Harvesting'),
                   (6,
                    '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/unmerged-ALCARECOStreamPromptCalibProdALCAPROMPT',
                    '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/ExpressAlcaSkimwrite_StreamExpress_ALCARECOAlcaHarvestALCARECOStreamPromptCalibProd',
                    'AlcaHarvest',
                    'Harvesting'),
                   (9,
                    '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/unmerged-logArchive',
                    '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO/AlcaSkimLogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (4,
                    '/TestWorkload/Express/ExpressMergewrite_PrimaryDataset1_FEVT/merged-logArchive',
                    '/TestWorkload/Express/ExpressMergewrite_PrimaryDataset1_FEVT/Expresswrite_PrimaryDataset1_FEVTMergeLogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (14,
                    '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/ExpressMergewrite_StreamExpress_DQMIOEndOfRunDQMHarvestMerged/unmerged-logArchive',
                    '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/ExpressMergewrite_StreamExpress_DQMIOEndOfRunDQMHarvestMerged/ExpressMergewrite_StreamExpress_DQMIOMergedEndOfRunDQMHarvestLogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (16,
                    '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/ExpressMergewrite_StreamExpress_DQMIOPeriodicDQMHarvestMerged/unmerged-logArchive',
                    '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/ExpressMergewrite_StreamExpress_DQMIOPeriodicDQMHarvestMerged/ExpressMergewrite_StreamExpress_DQMIOMergedPeriodicDQMHarvestLogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (17,
                    '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/merged-logArchive',
                    '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/Expresswrite_StreamExpress_DQMIOMergeLogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (13,
                    '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/merged-MergedDQMIO',
                    '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/ExpressMergewrite_StreamExpress_DQMIOEndOfRunDQMHarvestMerged',
                    'Harvest',
                    'Harvesting'),
                   (15,
                    '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/merged-MergedDQMIO',
                    '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO/ExpressMergewrite_StreamExpress_DQMIOPeriodicDQMHarvestMerged',
                    'Harvest',
                    'Harvesting'),
                   (18,
                    '/TestWorkload/Express/unmerged-logArchive',
                    '/TestWorkload/Express/ExpressLogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (2,
                    '/TestWorkload/Express/unmerged-write_PrimaryDataset1_FEVTFEVT',
                    '/TestWorkload/Express/ExpressCleanupUnmergedwrite_PrimaryDataset1_FEVT',
                    'SiblingProcessingBased',
                    'Cleanup'),
                   (3,
                    '/TestWorkload/Express/unmerged-write_PrimaryDataset1_FEVTFEVT',
                    '/TestWorkload/Express/ExpressMergewrite_PrimaryDataset1_FEVT',
                    'ExpressMerge',
                    'Merge'),
                   (5,
                    '/TestWorkload/Express/unmerged-write_StreamExpress_ALCARECOALCARECO',
                    '/TestWorkload/Express/ExpressAlcaSkimwrite_StreamExpress_ALCARECO',
                    'ExpressMerge',
                    'Express'),
                   (10,
                    '/TestWorkload/Express/unmerged-write_StreamExpress_ALCARECOALCARECO',
                    '/TestWorkload/Express/ExpressCleanupUnmergedwrite_StreamExpress_ALCARECO',
                    'SiblingProcessingBased',
                    'Cleanup'),
                   (11,
                    '/TestWorkload/Express/unmerged-write_StreamExpress_DQMIODQMIO',
                    '/TestWorkload/Express/ExpressCleanupUnmergedwrite_StreamExpress_DQMIO',
                    'SiblingProcessingBased',
                    'Cleanup'),
                   (12,
                    '/TestWorkload/Express/unmerged-write_StreamExpress_DQMIODQMIO',
                    '/TestWorkload/Express/ExpressMergewrite_StreamExpress_DQMIO',
                    'ExpressMerge',
                    'Merge'),
                   (1,
                    'TestWorkload-Express-Run123456',
                    '/TestWorkload/Express',
                    'Express',
                    'Express')]

        testArguments = ExpressWorkloadFactory.getTestArguments()
        testArguments.update(deepcopy(REQUEST))

        factory = ExpressWorkloadFactory()
        testWorkload = factory.factoryWorkloadConstruction("TestWorkload", testArguments)

        testWMBSHelper = WMBSHelper(testWorkload, "Express",
                                    blockName="Run123456",
                                    cachepath=self.testInit.testDir)
        testWMBSHelper.createTopLevelFileset()
        testWMBSHelper._createSubscriptionsInWMBS(testWMBSHelper.topLevelTask, testWMBSHelper.topLevelFileset)

        self.assertItemsEqual(testWorkload.listOutputProducingTasks(), expOutTasks)

        workflows = self.listTasksByWorkflow.execute(workflow="TestWorkload")
        self.assertItemsEqual([item['task'] for item in workflows], expWfTasks)

        # returns a tuple of id, name, open and last_update
        filesets = self.listFilesets.execute()
        self.assertItemsEqual([item[1] for item in filesets], expFsets)

        subscriptions = self.listSubsMapping.execute(workflow="TestWorkload", returnTuple=True)
        self.assertItemsEqual(subscriptions, subMaps)


if __name__ == '__main__':
    unittest.main()



import logging

class TestRegistryFile2:


    def doSomething(self, parameters = {}):
        logging.debug("TestCollection_2.TestRegistryFile2 is doing something")
        logging.debug("With parameters: "+str(parameters))
        return parameters



#!/usr/bin/env python
"""
_Template_

Unittest for WMCore.WMSpec.Steps.Template module

"""

import unittest

from WMCore.WMSpec.WMStep import WMStep, WMStepHelper
from WMCore.WMSpec.Steps.Template import Template, CoreHelper



class TemplateTest(unittest.TestCase):
    """
    TestCase for Template base object

    """

    def testA(self):
        """instantiation"""
        try:
            template = Template()
        except Exception as ex:
            msg = "Failed to instantiate Template object:\n"
            msg += str(ex)
            self.fail(msg)

    def testB(self):
        """test coreInstall method"""
        step = WMStep("TestStep")

        template = Template()

        try:
            template.coreInstall(step)
        except Exception as ex:
            msg = "Error calling Template.coreInstall(step)\n"
            msg += str(ex)
            self.fail(msg)

        # check environment installation
        self.assertTrue(getattr(step, "environment", None) != None)
        env = getattr(step, "environment")
        self.assertTrue(getattr(env, "paths", None) != None)
        self.assertTrue(getattr(env, "variables", None) != None)


        # check build installation
        self.assertTrue(getattr(step, "build", None) != None)
        build = getattr(step, "build")
        self.assertTrue(getattr(build, "directories", None) != None)

    def testC(self):
        """exceptions"""

        template = Template()
        step = WMStep("TestStep")

        self.assertRaises(NotImplementedError, template.install, step)
        self.assertRaises(NotImplementedError, template.helper, step)



class CoreHelperTest(unittest.TestCase):
    """
    TestCase for CoreHelper object
    """

    def testA(self):
        """instantiation"""
        try:
            helper = CoreHelper(WMStep("TestHelper"))
        except Exception as ex:
            msg = "Failed to instantiate CoreHelper object:\n"
            msg += str(ex)
            self.fail(msg)

    def testB(self):
        """test stepName"""
        stepName = "StepNameGoesHere"
        helper = CoreHelper(WMStep(stepName))

        try:
            name = helper.stepName()
        except Exception as ex:
            msg = "Failed to call CoreHelper.stepName:\n"
            msg += str(ex)
            self.fail(msg)

        self.assertEqual(name, stepName)


    def testC(self):
        """environment setting manipulators"""

        step = WMStep("CoreHelperTest")
        template = Template()
        template.coreInstall(step)
        helper = CoreHelper(step)

        try:
            helper.addEnvironmentVariable("Variable1", "Value1")
        except Exception as ex:
            msg = "Failed to call CoreHelper.addEnvironmentVariable:\n"
            msg += str(ex)
            self.fail(msg)

        env = helper.environment()
        self.assertTrue(getattr(env.variables, "Variable1", None) != None)
        self.assertEqual(getattr(env.variables, "Variable1"), "Value1")



        helper.addEnvironmentVariable("Variable2", "Value2")
        self.assertTrue(getattr(env.variables, "Variable2", None) != None)
        self.assertEqual(getattr(env.variables, "Variable2"), "Value2")

        try:
            helper.addEnvironmentPath("Path1", "Entry1")
            helper.addEnvironmentPath("Path1", "Entry2")
            helper.addEnvironmentPath("Path1", "Entry3")
        except Exception as ex:
            msg = "Failed to call CoreHelper.addEnvironmentPath:\n"
            msg += str(ex)
            self.fail(msg)


        path1 = getattr(env.paths, "Path1", None)
        self.assertTrue(path1 != None)
        self.assertEqual(path1, ['Entry1', 'Entry2', 'Entry3'])








    def testD(self):
        """build/dir/file structure manipulators"""
        step = WMStep("CoreHelperTest")
        template = Template()
        template.coreInstall(step)
        helper = CoreHelper(step)

        try:
            helper.addDirectory("dir1")
            helper.addDirectory("dir1/dir2")
            helper.addDirectory("dir1/dir3")
            helper.addDirectory("dir1/dir4")
            helper.addDirectory("/dir1/dir5")
        except Exception as ex:
            msg = "Error calling CoreHelper.addDirectory\n"
            msg += str(ex)
            self.fail(msg)

        dirs = helper.directoryStructure()
        self.assertTrue(hasattr(dirs, helper.stepName()))

        stepDir = getattr(dirs, helper.stepName())
        self.assertTrue(hasattr(stepDir, "dir1"))
        dir1 = getattr(stepDir, "dir1")
        dir2 = getattr(dir1, "dir2")

        self.assertTrue(hasattr(dir1, "dir2"))
        self.assertTrue(hasattr(dir1, "dir3"))
        self.assertTrue(hasattr(dir1, "dir4"))
        self.assertTrue(hasattr(dir1, "dir5"))



        helper.addFile("file1")
        helper.addFile("file2", "dir1/dir2/file2")

        self.assertTrue(hasattr(stepDir, "file1"))
        self.assertTrue(hasattr(dir2, "file2"))

        file1 = getattr(stepDir, "file1")
        file2 = getattr(dir2, "file2")

        self.assertEqual(file1['Source'], "file1")
        self.assertEqual(file1['Target'], "file1")

        self.assertEqual(file2['Source'], "file2")
        self.assertEqual(file2['Target'], "file2")






if __name__ == '__main__':
    unittest.main()



from __future__ import print_function

from builtins import range
from future import standard_library
standard_library.install_aliases()

import unittest
import time
from http.client import HTTPException

from WMCore_t.ReqMgr_t.TestConfig import config
from nose.plugins.attrib import attr

from Utils.PythonVersion import PY3

import WMCore
from WMQuality.REST.RESTBaseUnitTestWithDBBackend import RESTBaseUnitTestWithDBBackend


class AuxiliaryTest(RESTBaseUnitTestWithDBBackend):
    def setUp(self):
        config.main.tools.cms_auth.policy = "dangerously_insecure"
        self.setConfig(config)
        self.setCouchDBs([(config.views.data.couch_reqmgr_db, "ReqMgr"),
                          (config.views.data.couch_reqmgr_aux_db, "ReqMgrAux")])
        self.setSchemaModules([])
        RESTBaseUnitTestWithDBBackend.setUp(self)
        if PY3:
            self.assertItemsEqual = self.assertCountEqual

    def tearDown(self):
        RESTBaseUnitTestWithDBBackend.tearDown(self)

    def testProcStatus(self):
        """Test the `proc_status` REST API"""
        res = self.jsonSender.get("data/proc_status")
        self.assertItemsEqual(list(res[0]["result"][0]), ['server'])

    def testInfo(self):
        """Test the `info` REST API"""
        res = self.jsonSender.get("data/info")
        self.assertEqual(res[0]["result"][0]['wmcore_reqmgr_version'], WMCore.__version__)

    def testAbout(self):
        """Test the `about` REST API"""
        res = self.jsonSender.get("data/about")
        self.assertEqual(res[0]["result"][0]['wmcore_reqmgr_version'], WMCore.__version__)

    def testStatus(self):
        """Test the `status` REST API"""
        res = self.jsonSender.get("data/status")
        self.assertTrue(len(res[0]['result']) == 19)
        for st in {"aborted-archived", "new", "completed"}:
            self.assertIn(st, res[0]['result'])

    def testType(self):
        """Test the `type` REST API"""
        res = self.jsonSender.get("data/type")
        self.assertItemsEqual(res[0]["result"],
                              ['ReReco', 'StoreResults', 'Resubmission', 'TaskChain', 'DQMHarvest', 'StepChain'])

    def testPermissions(self):
        """Test the `permissions` REST API"""
        # no document yet available
        with self.assertRaises(HTTPException):
            self.jsonSender.get("data/permissions")

    def testCMSSWVersions(self):
        """Test the `cmsswversions` REST API"""
        myDoc = {"slc6": ["CMS1", "CMS2", "CMS3"], "slc7": ["CMS3", "CMS4"]}
        res = self.jsonSender.post("data/cmsswversions", myDoc)
        self.assertTrue(res)

        res = self.jsonSender.get("data/cmsswversions")
        self.assertTrue(res[0]["result"][0]["ConfigType"] == "CMSSW_VERSIONS")
        self.assertItemsEqual(list(res[0]["result"][0]), ["slc6", "slc7", "ConfigType"])

    def testPutCMSSWVersions(self):
        """Test the `cmsswversions` REST API with PUT call"""
        myDoc = {"slc6": ["CMS1", "CMS2", "CMS3"]}
        res = self.jsonSender.put("data/cmsswversions", myDoc)
        self.assertTrue(res)

        res = self.jsonSender.get("data/cmsswversions")
        self.assertItemsEqual(list(res[0]["result"][0]), ["slc6", "ConfigType"])

    def testWMAgentConfig(self):
        """Test the `wmagentconfig` REST API"""
        docName = "wmagent1"
        myDoc = {"key1": "blah"}
        res = self.jsonSender.post("data/wmagentconfig/%s" % docName, myDoc)
        self.assertTrue(res)

        res = self.jsonSender.get("data/wmagentconfig/%s" % docName)
        self.assertTrue(res[0]["result"][0]["ConfigType"] == "WMAGENT_CONFIG")
        self.assertItemsEqual(list(res[0]["result"][0]), ["key1", "ConfigType"])

    @attr("integration")
    def testPutWMAgentConfig(self):
        """
        Same as testWMAgentConfig, but test the PUT call in a different unit
        test because jenkins are not happy to run those
        """
        self.testWMAgentConfig()

        # now change the document with a PUT call
        docName = "wmagent1"
        myDoc = {"key2": ["blah"], "nono": False}
        res = self.jsonSender.put("data/wmagentconfig/%s" % docName, myDoc)
        self.assertTrue(res)

        res = self.jsonSender.get("data/wmagentconfig/%s" % docName)
        self.assertTrue(res[0]["result"][0]["ConfigType"] == "WMAGENT_CONFIG")
        self.assertItemsEqual(list(res[0]["result"][0]), ["key2", "nono", "ConfigType"])

    def testCampaignConfig(self):
        """Test the `campaignconfig` REST API"""
        docName = "camp1"
        myDoc = {"campName": "Camp1", "keyblah": False}
        res = self.jsonSender.post("data/campaignconfig/%s" % docName, myDoc)
        self.assertTrue(res)

        res = self.jsonSender.get("data/campaignconfig/%s" % docName)
        self.assertTrue(res[0]["result"][0]["ConfigType"] == "CAMPAIGN_CONFIG")
        self.assertItemsEqual(list(res[0]["result"][0]), ["campName", "keyblah", "ConfigType"])

    @attr("integration")
    def testPutCampaignConfig(self):
        """
        Same as testCampaignConfig, but test the PUT call in a different unit
        test because jenkins are not happy to run those
        """
        self.testCampaignConfig()

        # now change the document with a PUT call
        docName = "camp1"
        myDoc = {"campName": "NewCamp1", "keyblah": True}
        res = self.jsonSender.put("data/campaignconfig/%s" % docName, myDoc)
        self.assertTrue(res)

        res = self.jsonSender.get("data/campaignconfig/%s" % docName)
        self.assertTrue(res[0]["result"][0]["ConfigType"] == "CAMPAIGN_CONFIG")
        self.assertItemsEqual(list(res[0]["result"][0]), ["campName", "keyblah", "ConfigType"])

    def testUnifiedConfig(self):
        """Test the `unifiedconfig` REST API"""
        docName = "uni1"
        myDoc = {"key1": "value1"}
        res = self.jsonSender.post("data/unifiedconfig/%s" % docName, myDoc)
        self.assertTrue(res)

        res = self.jsonSender.get("data/unifiedconfig/%s" % docName)
        self.assertTrue(res[0]["result"][0]["ConfigType"] == "UNIFIED_CONFIG")
        self.assertItemsEqual(list(res[0]["result"][0]), ["key1", "ConfigType"])

    @attr("integration")
    def testPutUnifiedConfig(self):
        """
        Same as testUnifiedConfig, but test the PUT call in a different unit
        test because jenkins are not happy to run those
        """
        self.testUnifiedConfig()

        # now change the document with a PUT call
        docName = "uni1"
        myDoc = {"key2": ["blah"], "key3": True}
        res = self.jsonSender.put("data/unifiedconfig/%s" % docName, myDoc)
        self.assertTrue(res)

        res = self.jsonSender.get("data/unifiedconfig/%s" % docName)
        self.assertItemsEqual(list(res[0]["result"][0]), ["key2", "key3", "ConfigType"])

    def testTransferInfo(self):
        """Test the `transferinfo` REST API"""
        docName = "trans1"
        myDoc = {"key1": "value1"}
        res = self.jsonSender.post("data/transferinfo/%s" % docName, myDoc)
        self.assertTrue(res)

        res = self.jsonSender.get("data/transferinfo/%s" % docName)
        self.assertTrue(res[0]["result"][0]["ConfigType"] == "TRANSFER")
        self.assertItemsEqual(list(res[0]["result"][0]), ["key1", "ConfigType"])

    @attr("integration")
    def testPutTransferInfo(self):
        """
        Same as testTransferInfo, but test the PUT call in a different unit
        test because jenkins are not happy to run those
        """
        self.testTransferInfo()

        # now change the document with a PUT call
        docName = "trans1"
        myDoc = {"key2": ["blah"], "key3": True}
        res = self.jsonSender.put("data/transferinfo/%s" % docName, myDoc)
        self.assertTrue(res)

        res = self.jsonSender.get("data/transferinfo/%s" % docName)
        self.assertItemsEqual(list(res[0]["result"][0]), ["key2", "key3", "ConfigType"])

    def testAllTransferDocs(self):
        """Test the `transferinfo` REST API"""
        self.testTransferInfo()
        ### views not built/indexed yet
        res = self.jsonSender.get("data/transferinfo/ALL_DOCS")
        res = self.jsonSender.get("data/transferinfo/ALL_DOCS")
        if not res[0]["result"]:
            res = self.jsonSender.get("data/transferinfo/ALL_DOCS")
        self.assertEqual(len(res[0]["result"]), 1)

        docName = "trans2"
        myDoc = {"key_transf2": "transfer2"}
        res = self.jsonSender.post("data/transferinfo/%s" % docName, myDoc)
        self.assertTrue(res)

        res = self.jsonSender.get("data/transferinfo/ALL_DOCS")  # views still to be updated...
        self.assertEqual(len(res[0]["result"]), 1)

        # trick to make sure views have been updated
        for i in range(3):
            try:
                res = self.jsonSender.get("data/transferinfo/ALL_DOCS")
                self.assertEqual(len(res[0]["result"]), 2)
            except AssertionError:
                if i == 2:
                    raise
                else:
                    print("waiting 1 second for views to be updated...")
                    time.sleep(1)


if __name__ == "__main__":
    unittest.main()



"""
_GetParentInfo_

Figure out parentage information for a file in WMBS.  This will return
information about a file's parent such as the
lfn, id and whether or not the file is merged.
"""




from WMCore.Database.DBFormatter import DBFormatter

class GetParentInfo(DBFormatter):
    sql = """SELECT wfp.id, wfp.lfn, wfp.merged
             FROM wmbs_file_details wfp
             INNER JOIN wmbs_file_parent wfpa ON wfpa.parent = wfp.id
             INNER JOIN wmbs_file_details wfd ON wfd.id = wfpa.child
             WHERE wfd.lfn = :child_lfn
    """

    def execute(self, childLFNs, conn=None, transaction=False):
        bindVars = []
        for childLFN in childLFNs:
            bindVars.append({"child_lfn": childLFN})

        result = self.dbi.processData(self.sql, bindVars,
                         conn=conn, transaction=transaction)
        return self.formatDict(result)



#!/usr/bin/env python
"""
_TrivialFileCatalog_t_

Test the parsing of the TFC.
"""

from builtins import str

import os
import unittest
import tempfile

from WMCore.WMBase import getTestBase

from WMCore.Storage.TrivialFileCatalog import readTFC, TrivialFileCatalog


class TrivialFileCatalogTest(unittest.TestCase):
    def setUp(self):
        pass


    def tearDown(self):
        pass


    def testTrivialFileCatalog(self):
        """
        Run some simple tests on reading a trivialFileCatalog

        """
        tfcFilename = os.path.join(getTestBase(),
                                   "WMCore_t/Storage_t",
                                   "T1_US_FNAL_TrivialFileCatalog.xml")

        if not os.path.exists(tfcFilename):
            raise Exception("No TrivialFileCatalog found!")


        tfcInstance = readTFC(tfcFilename)

        self.assertEqual(type(tfcInstance), type(TrivialFileCatalog()))

        # Look for similarities in each node of the TFC file
        for mapping in ['lfn-to-pfn', 'pfn-to-lfn']:
            for x in tfcInstance[mapping]:
                self.assertEqual('path-match-expr' in x, True)
                self.assertEqual('path-match' in x, True)
                self.assertEqual('protocol' in x, True)
                self.assertEqual('result' in x, True)
                self.assertEqual('chain' in x, True)
                self.assertEqual(x['protocol'] in ['direct', 'dcap', 'srm', 'srmv2'],
                                 True, 'Could not find protocol %s' % (x['protocol']))
                self.assertEqual(x['chain'], None, 'Invalid chain %s' % (x['chain']))


    def testRoundTrip(self):
        """
        Test PFN, LFN matching upon an existing TFC XML file

        """
        tfc_file = os.path.join(getTestBase(),
                                "WMCore_t/Storage_t",
                                "T1_US_FNAL_TrivialFileCatalog.xml")
        tfc = readTFC(tfc_file)

        # Check that an lfn goes to an srmv2 pfn and comes back as the same lfn
        in_lfn = '/store/data/my/data'
        in_pfn = "srm://cmssrm.fnal.gov:8443/srm/managerv2?SFN=/11/store/data/my/data"
        out_pfn = tfc.matchLFN('srmv2', in_lfn)
        self.assertEqual(out_pfn, in_pfn)
        out_lfn = tfc.matchPFN('srmv2', in_pfn)
        self.assertEqual(in_lfn, out_lfn)
        out_pfn = tfc.matchLFN('srmv2', out_lfn)
        self.assertEqual(in_pfn, out_pfn)


    def testRoundTripWithChain(self):
        """
        Test PFN, LFN conversion when rules are chained.

        """
        tfc = TrivialFileCatalog()

        match = "/+(.*)"
        result = "/castor/cern.ch/cms/$1"
        tfc.addMapping("direct", match, result, mapping_type = "lfn-to-pfn")

        match = "(.*)"
        result = "$1"
        tfc.addMapping("stageout", match, result, chain = "direct",
                       mapping_type = "lfn-to-pfn")

        in_lfn = "/this/is/a/test/lfn"
        in_pfn = "/castor/cern.ch/cms" + in_lfn

        out_pfn = tfc.matchLFN("stageout", in_lfn)
        self.assertNotEqual(out_pfn, in_lfn)
        self.assertEqual(out_pfn, in_pfn)

        tfc = TrivialFileCatalog()

        match = "/+castor/cern\.ch/cms/(.*)"
        result = "/$1"
        tfc.addMapping("direct", match, result, mapping_type = "pfn-to-lfn")

        match = "(.*)"
        result = "$1"
        tfc.addMapping("stageout", match, result, chain = "direct",
                       mapping_type = "pfn-to-lfn")

        out_lfn = tfc.matchPFN("stageout", in_pfn)
        self.assertEqual(out_lfn, in_lfn)

    def testAddMapping(self):
        tfc = TrivialFileCatalog()
        lfn = "some_lfn"
        pfn = "some_pfn"
        tfc.addMapping("direct", lfn, pfn, mapping_type = 'lfn-to-pfn')
        tfc.addMapping("direct", pfn, lfn, mapping_type = 'pfn-to-lfn')
        out_pfn = tfc.matchLFN("direct", lfn)
        out_lfn = tfc.matchPFN("direct", pfn)
        self.assertEqual(lfn, out_lfn, "Error: incorrect matching")
        self.assertEqual(pfn, out_pfn, "Error: incorrect matching")


    def testSaveTFC(self):
        tfc = TrivialFileCatalog()
        lfn = "some_lfn"
        pfn = "some_pfn"
        tfc.addMapping("direct", lfn, pfn, mapping_type = 'lfn-to-pfn')
        tfc.addMapping("direct", pfn, lfn, mapping_type = 'pfn-to-lfn')

        f = tempfile.NamedTemporaryFile("w+", delete = True) # read / write
        tfcStr = str(tfc.getXML())
        f.write(tfcStr)
        f.flush()
        f.seek(0)

        tfcInstance = readTFC(f.name)
        out_pfn = tfc.matchLFN("direct", lfn)
        out_lfn = tfc.matchPFN("direct", pfn)
        self.assertEqual(lfn, out_lfn, "Error: incorrect matching")
        self.assertEqual(pfn, out_pfn, "Error: incorrect matching")
        f.close()

    def testMultipleRegexMatch(self):
        # Check that an lfn is converted to the right pfn
        in_lfn = '/store/user/fred/data'

        tfc_file = os.path.join(getTestBase(),
                                "WMCore_t/Storage_t",
                                "T2_CH_CERNBOX_TrivialFileCatalog.xml")
        tfc = readTFC(tfc_file)
        out_pfn = "root://eosuser.cern.ch/eos/user/f/fred/data"
        pfn = tfc.matchLFN('srmv2', in_lfn)
        self.assertEqual(out_pfn, pfn)

        tfc_file = os.path.join(getTestBase(),
                                "WMCore_t/Storage_t",
                                "T2_PT_NCG_Lisbon_TrivialFileCatalog.xml")
        tfc = readTFC(tfc_file)
        out_pfn = "srm://srm01.ncg.ingrid.pt:8444/srm/managerv2?SFN=/cmst3/store/user/fred/data"
        pfn = tfc.matchLFN('srmv2', in_lfn)
        self.assertEqual(out_pfn, pfn)

        tfc_file = os.path.join(getTestBase(),
                                "WMCore_t/Storage_t",
                                "T2_US_Florida_TrivialFileCatalog.xml")
        tfc = readTFC(tfc_file)
        out_pfn = "srm://srm.ihepa.ufl.edu:8443/srm/v2/server?SFN=/cms/data/store/user/fred/data"
        pfn = tfc.matchLFN('srmv2', in_lfn)
        self.assertEqual(out_pfn, pfn)

        tfc_file = os.path.join(getTestBase(),
                                "WMCore_t/Storage_t",
                                "T2_ES_IFCA_TrivialFileCatalog.xml")
        tfc = readTFC(tfc_file)
        out_pfn = "srm://srm01.ifca.es:8444/srm/managerv2?SFN=/cms/store/user/fred/data"
        pfn = tfc.matchLFN('srmv2', in_lfn)
        self.assertEqual(out_pfn, pfn)

        tfc_file = os.path.join(getTestBase(),
                                "WMCore_t/Storage_t",
                                "T2_US_Nebraska_TrivialFileCatalog.xml")
        tfc = readTFC(tfc_file)
        out_pfn = "srm://dcache07.unl.edu:8443/srm/v2/server?SFN=/mnt/hadoop/user/uscms01/pnfs/unl.edu/data4/cms/store/user/fred/data"
        pfn = tfc.matchLFN('srmv2', in_lfn)
        self.assertEqual(out_pfn, pfn)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
_DeleteCheckFileset_

Oracle implementation of DeleteCheckFileset

"""
__all__ = []



from WMCore.WMBS.MySQL.Fileset.DeleteCheck import DeleteCheck as MySQLDeleteCheck

class DeleteCheck(MySQLDeleteCheck):
    """
    Same as MySQL version

    """



#!/usr/bin/env python
"""
Unittests for IteratorTools functions
"""

from __future__ import division, print_function

import unittest
from copy import deepcopy

from Utils.PythonVersion import PY3

from WMCore.ReqMgr.DataStructs.Request import initialize_clone, RequestInfo
from WMCore.WMSpec.StdSpecs.ReReco import ReRecoWorkloadFactory
from WMCore.WMSpec.StdSpecs.StepChain import StepChainWorkloadFactory
from WMCore.WMSpec.StdSpecs.TaskChain import TaskChainWorkloadFactory
from WMCore.WMSpec.WMSpecErrors import WMSpecFactoryException

### Spec arguments definition with only key and its default value
RERECO_ARGS = ReRecoWorkloadFactory.getWorkloadCreateArgs()
STEPCHAIN_ARGS = StepChainWorkloadFactory.getWorkloadCreateArgs()
TASKCHAIN_ARGS = TaskChainWorkloadFactory.getWorkloadCreateArgs()
# inner Task/Step definition
STEP_ARGS = StepChainWorkloadFactory.getChainCreateArgs()
TASK_ARGS = TaskChainWorkloadFactory.getChainCreateArgs()

### Some original request dicts (ones to be cloned from)
rerecoOriginalArgs = {'Memory': 234, 'SkimName1': 'skim_2017', 'SkimInput1': 'RECOoutput',
                      'Skim1ConfigCacheID': 'abcdef', 'IncludeParents': True,
                      'TimePerEvent': 1.2, 'RequestType': 'ReReco', 'RequestName': 'test_rereco'}
stepChainOriginalArgs = {'Memory': 1234, 'TimePerEvent': 1.2, 'RequestType': 'StepChain',
                         "ScramArch": ["slc6_amd64_gcc481"], "RequestName": "test_stepchain",
                         "Step1": {"ConfigCacheID": "blah", "GlobalTag": "PHYS18", "BlockWhitelist": ["A", "B"],
                                   "ScramArch": ["slc6_amd64_gcc493"]},
                         "Step2": {"AcquisitionEra": "ACQERA", "ProcessingVersion": 3, "LumisPerJob": 10,
                                   "ScramArch": ["slc7_amd64_gcc630"]},
                         "StepChain": 2, "ConfigCacheID": None}
taskChainOriginalArgs = {'PrepID': None, 'Campaign': "MainTask", 'RequestType': 'TaskChain',
                         "ScramArch": ["slc6_amd64_gcc481", "slc7_amd64_gcc630"], "RequestName": "test_taskchain",
                         "Task1": {"ConfigCacheID": "blah", "InputDataset": "/1/2/RAW", "BlockWhitelist": ["A", "B"],
                                   "LumiList": {"202205": [[1, 10], [20, 25]], "202209": [[1, 3], [5, 5], [6, 9]]}},
                         "Task2": {"AcquisitionEra": "ACQERA", "KeepOutput": False, "ProcessingVersion": 2,
                                   "LumisPerJob": 10, "ScramArch": ["slc6_amd64_gcc493"]},
                         "Task3": {"InputTask": "task111", "TaskName": "blah", "LumisPerJob": 10},
                         "TaskChain": 3, "DQMConfigCacheID": 'unidunite',
                         "ProcessingVersion": {"name1": 4, "name2": 5, "name3": 6}}


def updateDict(dictObj1, dictObj2):
    """
    Util for updating the dictObj1 with the context of dictObj2 and return the final dict
    """
    dictObj1.update(dictObj2)
    return dictObj1


class RequestTests(unittest.TestCase):
    """
    unittest for ReqMgr DataStructs Request functions.
    """

    def setUp(self):
        if PY3:
            self.assertItemsEqual = self.assertCountEqual

    def testInvalidKeys_initialize_clone(self):
        """
        Test some undefined/assignment user arguments in initialize_clone
        """
        # test with unknown key
        requestArgs = {'Multicore': 1, 'WRONG': 'Alan'}
        originalArgs = deepcopy(taskChainOriginalArgs)
        cloneArgs = initialize_clone(requestArgs, originalArgs, TASKCHAIN_ARGS, TASK_ARGS)
        with self.assertRaises(WMSpecFactoryException):
            TaskChainWorkloadFactory().factoryWorkloadConstruction(cloneArgs["RequestName"], cloneArgs)

        # test with assignment key
        requestArgs = {'Multicore': 1, 'TrustSitelists': False}
        originalArgs = deepcopy(taskChainOriginalArgs)
        cloneArgs = initialize_clone(requestArgs, originalArgs, TASKCHAIN_ARGS, TASK_ARGS)
        with self.assertRaises(WMSpecFactoryException):
            TaskChainWorkloadFactory().factoryWorkloadConstruction(cloneArgs["RequestName"], cloneArgs)

        # test mix of StepChain and TaskChain args (it passes the initial filter but raises factory exception)
        requestArgs = {'SubRequestType': "RelVal", "Step2": {"LumisPerJob": 5, "Campaign": "Step2Camp"}}
        originalArgs = deepcopy(taskChainOriginalArgs)
        cloneArgs = initialize_clone(requestArgs, originalArgs, TASKCHAIN_ARGS, TASK_ARGS)
        with self.assertRaises(WMSpecFactoryException):
            TaskChainWorkloadFactory().factoryWorkloadConstruction(cloneArgs["RequestName"], cloneArgs)

        # make sure unsupported keys are remove from both top and inner dicts
        requestArgs = {}
        originalArgs = deepcopy(taskChainOriginalArgs)
        originalArgs['BAD'] = 'KEY'
        originalArgs['Task1']['SiteWhitelist'] = ['This_key_should_not_be_here']
        originalArgs['Task1']['Alan'] = False
        cloneArgs = initialize_clone(requestArgs, originalArgs, TASKCHAIN_ARGS, TASK_ARGS)
        self.assertFalse('BAD' in cloneArgs)
        self.assertFalse('SiteWhitelist' in cloneArgs['Task1'])
        self.assertFalse('Alan' in cloneArgs['Task1'])

    def testReReco_initialize_clone(self):
        """
        Test the basics of initialize_clone for a ReReco request. 
        """
        # test overwrite of original values
        requestArgs = {'TimePerEvent': 21, 'Multicore': 1, 'SkimName1': 'new_skim_name'}
        originalArgs = deepcopy(rerecoOriginalArgs)
        cloneArgs = initialize_clone(requestArgs, originalArgs, RERECO_ARGS)
        rerecoArgs = deepcopy(rerecoOriginalArgs)
        self.assertDictEqual(cloneArgs, updateDict(rerecoArgs, requestArgs))

        # test overwrite of values that are not in the original request
        requestArgs = {'DQMUploadProxy': 'test_file', 'RequestNumEvents': 1, 'DQMSequences': ['a', 'b']}
        originalArgs = deepcopy(rerecoOriginalArgs)
        cloneArgs = initialize_clone(requestArgs, originalArgs, RERECO_ARGS)
        rerecoArgs = deepcopy(rerecoOriginalArgs)
        self.assertDictEqual(cloneArgs, updateDict(rerecoArgs, requestArgs))

        # check whether invalid keys in the original request are dumped
        requestArgs = {'RequestNumEvents': 1, 'DQMSequences': []}
        originalArgs = deepcopy(rerecoOriginalArgs)
        originalArgs.update(badJob='whatever', WMCore={'YouFool': True})
        cloneArgs = initialize_clone(requestArgs, originalArgs, RERECO_ARGS)
        rerecoArgs = deepcopy(rerecoOriginalArgs)
        self.assertDictEqual(cloneArgs, updateDict(rerecoArgs, requestArgs))

    def testSC_initialize_clone(self):
        """
        Test initialize_clone handling StepChains
        """
        # test overwrite of original values
        requestArgs = {'TimePerEvent': 12.34, 'ConfigCacheID': 'couchid', 'StepChain': 20}
        originalArgs = deepcopy(stepChainOriginalArgs)
        cloneArgs = initialize_clone(requestArgs, originalArgs, STEPCHAIN_ARGS, STEP_ARGS)
        scArgs = deepcopy(stepChainOriginalArgs)
        scArgs['Step2']['ProcessingVersion'] += 1
        self.assertDictEqual(cloneArgs, updateDict(scArgs, requestArgs))

        # test overwrite of values that are not in the original request
        requestArgs = {'DQMUploadProxy': 'test_file', 'RequestNumEvents': 1, 'DQMSequences': ['a', 'b']}
        originalArgs = deepcopy(stepChainOriginalArgs)
        cloneArgs = initialize_clone(requestArgs, originalArgs, STEPCHAIN_ARGS, STEP_ARGS)
        scArgs = deepcopy(stepChainOriginalArgs)
        scArgs['Step2']['ProcessingVersion'] += 1
        self.assertDictEqual(cloneArgs, updateDict(scArgs, requestArgs))

        # check whether invalid keys in the original request are dumped
        requestArgs = {'RequestNumEvents': 1, 'DQMSequences': []}
        originalArgs = deepcopy(stepChainOriginalArgs)
        originalArgs.update(Chain='whatever', WMCore={'YouFool': True})
        cloneArgs = initialize_clone(requestArgs, originalArgs, STEPCHAIN_ARGS, STEP_ARGS)
        scArgs = deepcopy(stepChainOriginalArgs)
        scArgs['Step2']['ProcessingVersion'] += 1
        self.assertDictEqual(cloneArgs, updateDict(scArgs, requestArgs))

        # test overwrite of inner Step dictionaries. Make sure ProcVer gets the user provided value
        requestArgs = {'SubRequestType': "RelVal", "Step1": {"BlockWhitelist": ["C"]},
                       "Step2": {"LumisPerJob": 5, "Campaign": "Step2Camp", "ProcessingVersion": 1}}
        originalArgs = deepcopy(stepChainOriginalArgs)
        cloneArgs = initialize_clone(requestArgs, originalArgs, STEPCHAIN_ARGS, STEP_ARGS)
        scArgs = deepcopy(stepChainOriginalArgs)
        # setting by hand, I assume I better not use the code that is being tested here :-)
        updateDict(scArgs, requestArgs)
        scArgs['Step1'] = {"ConfigCacheID": "blah", "GlobalTag": "PHYS18",
                           "BlockWhitelist": ["C"], "ScramArch": ["slc6_amd64_gcc493"]}
        scArgs['Step2'] = {"AcquisitionEra": "ACQERA", "ProcessingVersion": 1, "LumisPerJob": 5,
                           "Campaign": "Step2Camp", "ScramArch": ["slc7_amd64_gcc630"]}
        self.assertDictEqual(cloneArgs, scArgs)

    def testTC_initialize_clone(self):
        """
        Test initialize_clone handling TaskChains
        """

        def updateProcVer(tcArgs):
            tcArgs['Task2']['ProcessingVersion'] += 1
            tcArgs['ProcessingVersion']['name1'] += 1
            tcArgs['ProcessingVersion']['name2'] += 1
            tcArgs['ProcessingVersion']['name3'] += 1

        # test overwrite of original values
        requestArgs = {'ScramArch': "slc6_amd64_gcc630", 'DQMConfigCacheID': None, 'TaskChain': 20}
        originalArgs = deepcopy(taskChainOriginalArgs)
        cloneArgs = initialize_clone(requestArgs, originalArgs, TASKCHAIN_ARGS, TASK_ARGS)
        tcArgs = deepcopy(taskChainOriginalArgs)
        updateProcVer(tcArgs)
        self.assertDictEqual(cloneArgs, updateDict(tcArgs, requestArgs))

        # test overwrite of values that are not in the original request
        requestArgs = {'Task4': {"InputTask": "task222", "TransientOutputModules": ["RAW", "AOD"]},
                       'ConfigCacheID': None, 'EventStreams': 8}
        originalArgs = deepcopy(taskChainOriginalArgs)
        cloneArgs = initialize_clone(requestArgs, originalArgs, TASKCHAIN_ARGS, TASK_ARGS)
        tcArgs = deepcopy(taskChainOriginalArgs)
        updateProcVer(tcArgs)
        self.assertDictEqual(cloneArgs, updateDict(tcArgs, requestArgs))

        # check whether invalid keys in the original request are dumped
        requestArgs = {'RequestPriority': 1, 'DQMSequences': [], "AcquisitionEra": {"T1": "a", "T2": "b"}}
        originalArgs = deepcopy(taskChainOriginalArgs)
        originalArgs.update(ioInput='whatever', WMCore={'YouFool': True}, TotalJobs=123)
        cloneArgs = initialize_clone(requestArgs, originalArgs, TASKCHAIN_ARGS, TASK_ARGS)
        tcArgs = deepcopy(taskChainOriginalArgs)
        updateProcVer(tcArgs)
        self.assertDictEqual(cloneArgs, updateDict(tcArgs, requestArgs))

        # test overwrite of inner Task dictionaries. Make sure ProcVer gets the user provided value
        requestArgs = {"Task1": {"LumisPerJob": 1, "LumiList": {"202209": [[3, 5]]}},
                       "Task2": {"ProcessingVersion": 10},
                       "Task3": {"TaskName": "newBlah", "TransientOutputModules": []},
                       "PrepID": "prepBlah", "ProcessingVersion": 1}
        originalArgs = deepcopy(taskChainOriginalArgs)
        cloneArgs = initialize_clone(requestArgs, originalArgs, TASKCHAIN_ARGS, TASK_ARGS)
        tcArgs = deepcopy(taskChainOriginalArgs)
        # setting by hand, I assume I better not use the code that is being tested here :-)
        tcArgs['Task1'].update(requestArgs['Task1'])
        tcArgs['Task1']['LumiList'] = {"202205": [[1, 10], [20, 25]], "202209": [[3, 5]]}
        tcArgs['Task2'].update(requestArgs['Task2'])
        tcArgs['Task3'].update(requestArgs['Task3'])
        tcArgs.update({"PrepID": "prepBlah", "ProcessingVersion": 1})
        self.assertDictEqual(cloneArgs, tcArgs)

    def testRequestInfoReReco(self):
        """
        Test the RequestInfo class with a ReReco workflow.
        """
        # test overwrite of original values
        reqArgs = deepcopy(rerecoOriginalArgs)
        reqArgs['AgentJobInfo'] = {"agent_a": {"num_jobs": 2, "status": "done"}}
        reqInfo = RequestInfo(reqArgs)

        self.assertItemsEqual(reqArgs, reqInfo.data)

        # test `andFilterCheck` method
        self.assertFalse(reqInfo.andFilterCheck(dict(RequestType="BAD_TYPE")))
        self.assertTrue(reqInfo.andFilterCheck(dict(RequestType="ReReco")))
        self.assertFalse(reqInfo.andFilterCheck(dict(IncludeParents="False")))
        self.assertTrue(reqInfo.andFilterCheck(dict(IncludeParents="True")))
        self.assertTrue(reqInfo.andFilterCheck(dict(IncludeParents=True)))
        self.assertFalse(reqInfo.andFilterCheck(dict(AgentJobInfo="CLEANED")))

        # test `isWorkflowCleaned` method
        self.assertFalse(reqInfo.isWorkflowCleaned())

        # finally, test the `get` method
        self.assertIsNone(reqInfo.get(prop="NonExistent", default=None))
        self.assertIsNone(reqInfo.get(prop="NonExistent"))
        self.assertEqual(reqInfo.get(prop="NonExistent", default="diff_default"), "diff_default")
        self.assertEqual(reqInfo.get(prop="Memory"), 234)

    def testRequestInfoStepChain(self):
        """
        Test the RequestInfo class with a StepChain workflow.
        """
        # test overwrite of original values
        reqArgs = deepcopy(stepChainOriginalArgs)
        reqArgs['AgentJobInfo'] = {"agent_a": {"num_jobs": 2, "status": "done"}}
        from pprint import pprint
        pprint(reqArgs)

        reqInfo = RequestInfo(reqArgs)
        self.assertItemsEqual(reqArgs, reqInfo.data)

        # test `andFilterCheck` method
        self.assertFalse(reqInfo.andFilterCheck(dict(RequestType="BAD_TYPE")))
        self.assertTrue(reqInfo.andFilterCheck(dict(RequestType="StepChain")))
        self.assertFalse(reqInfo.andFilterCheck(dict(IncludeParents="False")))
        self.assertFalse(reqInfo.andFilterCheck(dict(IncludeParents="True")))
        self.assertFalse(reqInfo.andFilterCheck(dict(IncludeParents=True)))
        self.assertTrue(reqInfo.andFilterCheck(dict(ProcessingVersion=3)))
        self.assertTrue(reqInfo.andFilterCheck(dict(GlobalTag="PHYS18")))
        self.assertFalse(reqInfo.andFilterCheck(dict(AgentJobInfo="CLEANED")))

        # test `isWorkflowCleaned` method
        self.assertFalse(reqInfo.isWorkflowCleaned())

        # finally, test the `get` method
        self.assertIsNone(reqInfo.get(prop="NonExistent", default=None))
        self.assertIsNone(reqInfo.get(prop="NonExistent"))
        self.assertEqual(reqInfo.get(prop="NonExistent", default="diff_default"), "diff_default")
        # Memory is not inside any Step dict, that's why a plain integer...
        self.assertEqual(reqInfo.get(prop="Memory"), 1234)
        self.assertEqual(reqInfo.get(prop="RequestName"), "test_stepchain")
        self.assertEqual(reqInfo.get(prop="AcquisitionEra"), ["ACQERA"])
        self.assertEqual(reqInfo.get(prop="LumisPerJob"), [10])
        # NOTE: inner step properties have precedence over the top level ones.
        self.assertItemsEqual(reqInfo.get(prop="ScramArch"), ["slc6_amd64_gcc493", "slc7_amd64_gcc630"])

    def testRequestInfoTaskChain(self):
        """
        Test the RequestInfo class with a TaskChain workflow.
        """
        # test overwrite of original values
        reqArgs = deepcopy(taskChainOriginalArgs)
        reqArgs['AgentJobInfo'] = {"agent_a": {}}
        from pprint import pprint
        pprint(reqArgs)

        reqInfo = RequestInfo(reqArgs)
        self.assertItemsEqual(reqArgs, reqInfo.data)

        # test `andFilterCheck` method
        self.assertFalse(reqInfo.andFilterCheck(dict(RequestType="BAD_TYPE")))
        self.assertTrue(reqInfo.andFilterCheck(dict(RequestType="TaskChain")))
        self.assertFalse(reqInfo.andFilterCheck(dict(IncludeParents="False")))
        self.assertFalse(reqInfo.andFilterCheck(dict(IncludeParents="True")))
        self.assertFalse(reqInfo.andFilterCheck(dict(IncludeParents=True)))
        self.assertTrue(reqInfo.andFilterCheck(dict(KeepOutput=False)))
        # NOTE: None value will always fail to match (thus returning False)
        self.assertFalse(reqInfo.andFilterCheck(dict(PrepID=None)))
        self.assertTrue(reqInfo.andFilterCheck(dict(TaskChain=3)))
        self.assertTrue(reqInfo.andFilterCheck(dict(Campaign="MainTask")))
        self.assertTrue(reqInfo.andFilterCheck(dict(AgentJobInfo="CLEANED")))

        # test `isWorkflowCleaned` method
        self.assertTrue(reqInfo.isWorkflowCleaned())

        # finally, test the `get` method
        self.assertIsNone(reqInfo.get(prop="NonExistent", default=None))
        self.assertIsNone(reqInfo.get(prop="NonExistent"))
        self.assertEqual(reqInfo.get(prop="NonExistent", default="diff_default"), "diff_default")
        # PrepID is not inside any Step dict, that's why a plain integer...
        self.assertEqual(reqInfo.get(prop="PrepID"), None)
        self.assertEqual(reqInfo.get(prop="RequestName"), "test_taskchain")
        self.assertEqual(reqInfo.get(prop="LumisPerJob"), [10])
        self.assertEqual(reqInfo.get(prop="AcquisitionEra"), ["ACQERA"])
        # NOTE: inner task properties have precedence over the top level ones.
        self.assertEqual(reqInfo.get(prop="ProcessingVersion"), [2])
        self.assertItemsEqual(reqInfo.get(prop="BlockWhitelist"), ["A", "B"])
        self.assertItemsEqual(reqInfo.get(prop="ScramArch"), ['slc6_amd64_gcc481',
                                                              'slc6_amd64_gcc493', 'slc7_amd64_gcc630'])


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
Script meant to fetch the number of blocks and files for a given dataset,
using both DBS and Rucio services. It prints any inconsistency among
those two.
"""
from __future__ import print_function, division

import logging
import sys
from collections import Counter

from future.utils import viewkeys, viewvalues

from WMCore.Services.DBS.DBS3Reader import DBS3Reader
from WMCore.Services.Rucio.Rucio import Rucio

RUCIO_ACCT = "wma_prod"
RUCIO_HOST = "http://cms-rucio.cern.ch"
RUCIO_AUTH = "https://cms-rucio-auth.cern.ch"
DBS_URL = "https://cmsweb-prod.cern.ch/dbs/prod/global/DBSReader"


def loggerSetup(logLevel=logging.INFO):
    """
    Return a logger which writes everything to stdout.
    """
    logger = logging.getLogger(__name__)
    outHandler = logging.StreamHandler(sys.stdout)
    outHandler.setFormatter(logging.Formatter("%(asctime)s:%(levelname)s:%(module)s: %(message)s"))
    outHandler.setLevel(logLevel)
    logger.addHandler(outHandler)
    logger.setLevel(logLevel)
    return logger


def getFromRucio(dataset, logger):
    """
    Using the WMCore Rucio object and fetch all the blocks and files
    for a given container.
    Returns a dictionary key'ed by the block name, value is the amount of files.
    """
    rucio = Rucio(acct=RUCIO_ACCT,
                  hostUrl=RUCIO_HOST,
                  authUrl=RUCIO_AUTH,
                  configDict={'logger': logger})

    result = dict()
    for block in rucio.getBlocksInContainer(dataset):
        data = rucio.getDID(block)
        result.setdefault(block, data['length'])
    return result


def getFromDBS(dataset, logger):
    """
    Uses the WMCore DBS3Reader object to fetch all the blocks and files
    for a given container.
    Returns a dictionary key'ed by the block name, and an inner dictionary
    with the number of valid and invalid files. It also returns a total counter
    for the number of valid and invalid files in the dataset.
    """
    dbsReader = DBS3Reader(DBS_URL, logger)

    result = dict()
    dbsFilesCounter = Counter({'valid': 0, 'invalid': 0})
    blocks = dbsReader.listFileBlocks(dataset)
    for block in blocks:
        data = dbsReader.dbs.listFileArray(block_name=block, validFileOnly=0, detail=True)
        result.setdefault(block, Counter({'valid': 0, 'invalid': 0}))
        for fileInfo in data:
            if fileInfo['is_file_valid'] == 1:
                result[block]['valid'] += 1
                dbsFilesCounter['valid'] += 1
            else:
                result[block]['invalid'] += 1
                dbsFilesCounter['invalid'] += 1
    return result, dbsFilesCounter


def main():
    """
    Expects a dataset name as input argument.
    It then queries Rucio and DBS and compare their blocks and
    number of files.
    """
    if len(sys.argv) != 2:
        print("A dataset name must be provided in the command line")
        sys.exit(1)
    datasetName = sys.argv[1]

    logger = loggerSetup(logging.INFO)

    rucioOutput = getFromRucio(datasetName, logger)
    dbsOutput, dbsFilesCounter = getFromDBS(datasetName, logger)

    logger.info("*** Dataset: %s", datasetName)
    logger.info("Rucio file count : %s", sum(viewvalues(rucioOutput)))
    logger.info("DBS file count   : %s", dbsFilesCounter['valid'] + dbsFilesCounter['invalid'])
    logger.info(" - valid files   : %s", dbsFilesCounter['valid'])
    logger.info(" - invalid files : %s", dbsFilesCounter['invalid'])
    logger.info("Blocks in Rucio but not in DBS: %s", set(viewkeys(rucioOutput)) - set(viewkeys(dbsOutput)))
    logger.info("Blocks in DBS but not in Rucio: %s", set(viewkeys(dbsOutput)) - set(viewkeys(rucioOutput)))

    for blockname in rucioOutput:
        if blockname not in dbsOutput:
            logger.error("This block does not exist in DBS: %s", blockname)
            continue
        if rucioOutput[blockname] != sum(viewvalues(dbsOutput[blockname])):
            logger.warning("Block with file mismatch: %s", blockname)
            logger.warning("\tRucio: %s\t\tDBS: %s", rucioOutput[blockname], sum(viewvalues(dbsOutput[blockname])))


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
"""
_StoreResults_t_

Unit tests for the StoreResults workflow.
"""
from __future__ import print_function
from future.utils import viewitems

import threading
import unittest

from Utils.PythonVersion import PY3

from WMCore.DAOFactory import DAOFactory
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow import Workflow
from WMCore.WMSpec.StdSpecs.StoreResults import StoreResultsWorkloadFactory
from WMCore.WorkQueue.WMBSHelper import WMBSHelper
from WMQuality.TestInit import TestInit


class StoreResultsTest(unittest.TestCase):
    def setUp(self):
        """
        _setUp_

        Initialize the database.
        """
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules=["WMCore.WMBS"],
                                useDefault=False)
        self.testDir = self.testInit.generateWorkDir()

        myThread = threading.currentThread()
        self.daoFactory = DAOFactory(package="WMCore.WMBS",
                                     logger=myThread.logger,
                                     dbinterface=myThread.dbi)
        self.listTasksByWorkflow = self.daoFactory(classname="Workflow.LoadFromName")
        self.listFilesets = self.daoFactory(classname="Fileset.List")
        self.listSubsMapping = self.daoFactory(classname="Subscriptions.ListSubsAndFilesetsFromWorkflow")
        if PY3:
            self.assertItemsEqual = self.assertCountEqual
        return

    def tearDown(self):
        """
        _tearDown_

        Clear out the database.
        """
        self.testInit.clearDatabase()
        self.testInit.delWorkDir()
        return

    def testStoreResults(self):
        """
        _testStoreResults_

        Create a StoreResults workflow and verify it installs into WMBS
        correctly.
        """
        arguments = StoreResultsWorkloadFactory.getTestArguments()

        factory = StoreResultsWorkloadFactory()
        testWorkload = factory.factoryWorkloadConstruction("TestWorkload", arguments)

        testWMBSHelper = WMBSHelper(testWorkload, "StoreResults", "SomeBlock", cachepath=self.testDir)
        testWMBSHelper.createTopLevelFileset()
        testWMBSHelper._createSubscriptionsInWMBS(testWMBSHelper.topLevelTask, testWMBSHelper.topLevelFileset)

        testWorkflow = Workflow(name="TestWorkload",
                                task="/TestWorkload/StoreResults")
        testWorkflow.load()

        self.assertEqual(len(testWorkflow.outputMap), 2,
                         "Error: Wrong number of WF outputs.")
        goldenOutputMods = {"Merged": "USER"}
        for goldenOutputMod, tier in viewitems(goldenOutputMods):
            fset = goldenOutputMod + tier
            mergedOutput = testWorkflow.outputMap[fset][0]["merged_output_fileset"]
            unmergedOutput = testWorkflow.outputMap[fset][0]["output_fileset"]

            mergedOutput.loadData()
            unmergedOutput.loadData()

            self.assertEqual(mergedOutput.name, "/TestWorkload/StoreResults/merged-%s" % fset,
                             "Error: Merged output fileset is wrong: %s" % mergedOutput.name)
            self.assertEqual(unmergedOutput.name, "/TestWorkload/StoreResults/merged-%s" % fset,
                             "Error: Unmerged output fileset is wrong: %s." % unmergedOutput.name)

        logArchOutput = testWorkflow.outputMap["logArchive"][0]["merged_output_fileset"]
        unmergedLogArchOutput = testWorkflow.outputMap["logArchive"][0]["output_fileset"]
        logArchOutput.loadData()
        unmergedLogArchOutput.loadData()

        self.assertEqual(logArchOutput.name, "/TestWorkload/StoreResults/merged-logArchive",
                         "Error: LogArchive output fileset is wrong.")
        self.assertEqual(unmergedLogArchOutput.name, "/TestWorkload/StoreResults/merged-logArchive",
                         "Error: LogArchive output fileset is wrong.")

        topLevelFileset = Fileset(name="TestWorkload-StoreResults-SomeBlock")
        topLevelFileset.loadData()

        procSubscription = Subscription(fileset=topLevelFileset, workflow=testWorkflow)
        procSubscription.loadData()

        self.assertEqual(procSubscription["type"], "Merge",
                         "Error: Wrong subscription type.")
        self.assertEqual(procSubscription["split_algo"], "ParentlessMergeBySize",
                         "Error: Wrong split algo.")

        return

    def testFilesets(self):
        """
        Test workflow tasks, filesets and subscriptions creation
        """
        # expected tasks, filesets, subscriptions, etc
        expOutTasks = ['/TestWorkload/StoreResults']
        expWfTasks = ['/TestWorkload/StoreResults',
                      '/TestWorkload/StoreResults/StoreResultsLogCollect']
        expFsets = ['TestWorkload-StoreResults-/MinimumBias/ComissioningHI-v1/RAW',
                    '/TestWorkload/StoreResults/merged-MergedUSER',
                    '/TestWorkload/StoreResults/merged-logArchive']
        subMaps = [(2,
                    '/TestWorkload/StoreResults/merged-logArchive',
                    '/TestWorkload/StoreResults/StoreResultsLogCollect',
                    'MinFileBased',
                    'LogCollect'),
                   (1,
                    'TestWorkload-StoreResults-/MinimumBias/ComissioningHI-v1/RAW',
                    '/TestWorkload/StoreResults',
                    'ParentlessMergeBySize',
                    'Merge')]

        testArguments = StoreResultsWorkloadFactory.getTestArguments()

        factory = StoreResultsWorkloadFactory()
        testWorkload = factory.factoryWorkloadConstruction("TestWorkload", testArguments)

        testWMBSHelper = WMBSHelper(testWorkload, "StoreResults", blockName=testArguments['InputDataset'],
                                    cachepath=self.testInit.testDir)
        testWMBSHelper.createTopLevelFileset()
        testWMBSHelper._createSubscriptionsInWMBS(testWMBSHelper.topLevelTask, testWMBSHelper.topLevelFileset)

        self.assertItemsEqual(testWorkload.listOutputProducingTasks(), expOutTasks)

        workflows = self.listTasksByWorkflow.execute(workflow="TestWorkload")
        self.assertItemsEqual([item['task'] for item in workflows], expWfTasks)

        # returns a tuple of id, name, open and last_update
        filesets = self.listFilesets.execute()
        self.assertItemsEqual([item[1] for item in filesets], expFsets)

        subscriptions = self.listSubsMapping.execute(workflow="TestWorkload", returnTuple=True)
        self.assertItemsEqual(subscriptions, subMaps)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_Mask_

Unittest for the WMCore.DataStructs.Mask class

"""

# This code written as essentially a blank for future
# Mask development
# -mnorman


import unittest

from WMCore.DataStructs.Mask import Mask
from WMCore.DataStructs.Run import Run


class MaskTest(unittest.TestCase):
    """
    _MaskTest_

    """

    def testSetMaxAndSkipEvents(self):
        """
        test class for setMaxAndSkipEvents in Mask.py

        """

        testMask = Mask()
        maxEvents = 100
        skipEvents = 10

        testMask.setMaxAndSkipEvents(maxEvents, skipEvents)

        self.assertEqual(testMask['FirstEvent'], skipEvents)
        self.assertEqual(testMask['LastEvent'], maxEvents + skipEvents)

        return

    def testSetMaxAndSkipLumis(self):
        """
        test class for setMaxAndSkipLumis in Mask.py

        """

        testMask = Mask()
        maxLumis = 10
        skipLumis = 2

        testMask.setMaxAndSkipLumis(maxLumis, skipLumis)

        self.assertEqual(testMask['FirstLumi'], skipLumis)
        self.assertEqual(testMask['LastLumi'], maxLumis + skipLumis)

        return

    def testSetMaxAndSkipRuns(self):
        """
        test class for setMaxAndSkipRuns in Mask.py

        """

        testMask = Mask()
        maxRuns = 1000
        skipRuns = 200

        testMask.setMaxAndSkipRuns(maxRuns, skipRuns)

        self.assertEqual(testMask['FirstRun'], skipRuns)
        self.assertEqual(testMask['LastRun'], maxRuns + skipRuns)

        return

    def testGetMaxEvents(self):
        """
        test class for getMaxEvents in Mask.py

        """

        # The way I've decided to implement this depends on SetMaxAndSkipEvents()
        # Therefore a failure in one will result in a failure in the second
        # I'm not sure if this is the best way, but it's the one users will use
        # The problem is that it's called in reverse order by unittest so you have to
        # remember that.
        # -mnorman

        testMask = Mask()
        maxEvents = 100
        skipEvents = 1

        self.assertEqual(testMask.getMaxEvents(), None)

        testMask.setMaxAndSkipEvents(maxEvents, skipEvents)

        self.assertEqual(testMask.getMaxEvents(), maxEvents + skipEvents)

    def testGetMax(self):
        """
        test class for the getMax() routine added to Mask.py

        """

        testMask = Mask()
        maxRuns = 999
        skipRuns = 201

        testMask.setMaxAndSkipRuns(maxRuns, skipRuns)
        self.assertEqual(testMask.getMax('Event'), None)
        self.assertEqual(testMask.getMax('Lumi'), None)
        self.assertEqual(testMask.getMax('junk'), None)
        self.assertEqual(testMask.getMax('Run'), 1000)

    def testRunsAndLumis(self):
        """
        Test several different ways of creating the same list
        of runs and lumis
        """

        runMask = Mask()
        rangesMask = Mask()
        runAndLumisMask = Mask()

        runMask.addRun(Run(100, 1, 2, 3, 4, 5, 6, 8, 9, 10))
        runMask.addRun(Run(200, 6, 7, 8))
        runMask.addRun(Run(300, 12))

        rangesMask.addRunWithLumiRanges(run=100, lumiList=[[1, 6], [8, 10]])
        rangesMask.addRunWithLumiRanges(run=200, lumiList=[[6, 8]])
        rangesMask.addRunWithLumiRanges(run=300, lumiList=[[12, 12]])

        runAndLumisMask.addRunAndLumis(run=100, lumis=[1, 6])
        runAndLumisMask.addRunAndLumis(run=100, lumis=[8, 10])
        runAndLumisMask.addRunAndLumis(run=200, lumis=[6, 8])
        runAndLumisMask.addRunAndLumis(run=300, lumis=[12, 12])

        self.assertEqual(runMask.getRunAndLumis(), rangesMask.getRunAndLumis())
        # Note, this may break if the TODO in Mask.addRunAndLumis() is addressed
        self.assertEqual(runMask.getRunAndLumis(), runAndLumisMask.getRunAndLumis())

    def testFilter(self):
        """
        Test filtering of a set(run) object
        """
        mask = Mask()
        mask.addRunWithLumiRanges(run=1, lumiList=[[1, 9], [12, 12], [31, 31], [38, 39], [49, 49]])

        runs = set()
        runs.add(Run(1, 148, 166, 185, 195, 203, 212))
        newRuns = mask.filterRunLumisByMask(runs=runs)
        self.assertEqual(len(newRuns), 0)

        runs = set()
        runs.add(Run(1, 2, 148, 166, 185, 195, 203, 212))
        runs.add(Run(2, 148, 166, 185, 195, 203, 212))
        newRuns = mask.filterRunLumisByMask(runs=runs)
        self.assertEqual(len(newRuns), 1)

        runs = set()
        runs.add(Run(1, 2, 9, 148, 166, 185, 195, 203, 212))
        newRuns = mask.filterRunLumisByMask(runs=runs)
        self.assertEqual(len(newRuns), 1)
        run = newRuns.pop()
        self.assertEqual(run.run, 1)
        self.assertEqual(run.lumis, [2, 9])

    def testFilterRealCase(self):
        """
        Test filtering of a set(run) object based on real cases from production
        """
        mask = Mask()
        mask.addRunWithLumiRanges(run=1, lumiList=[[9, 9], [8, 8], [3, 4], [7, 7]])
        mask.setMaxAndSkipLumis(0, 7)
        mask.setMaxAndSkipRuns(0, 1)

        runs = set()
        runs.add(Run(1, *[(9, 500), (10, 500)]))
        runs.add(Run(1, *[(3, 500), (4, 500)]))
        runs.add(Run(1, *[(7, 500), (8, 500)]))
        newRuns = mask.filterRunLumisByMask(runs=runs)
        self.assertEqual(len(newRuns), 1)
        run = newRuns.pop()
        self.assertEqual(run.run, 1)
        self.assertEqual(run.lumis, [3, 4, 7, 8, 9])


if __name__ == '__main__':
    unittest.main()
